searchData={"items":[{"type":"module","title":"Ratatouille","doc":"Ratatouille is a framework for building terminal UIs.","ref":"Ratatouille.html"},{"type":"function","title":"Ratatouille.run/2","doc":"Starts an application under a supervised runtime, given a module implementing\nthe `Ratatouille.Application` behaviour. This call blocks until the\napplication is quit (or it crashes).\n\nThis is intended as a way to easily run simple apps and examples. For more\ncomplex apps depending on other processes, it's recommended to define an OTP\napplication and start the `Ratatouille.Runtime.Supervisor` as a child of your\napplication supervisor.","ref":"Ratatouille.html#run/2"},{"type":"function","title":"Example - Ratatouille.run/2","doc":"defmodule Counter do\n      @behaviour Ratatouille.App\n\n      import Ratatouille.View\n\n      def init(_context), do: 0\n\n      def update(model, msg) do\n        case msg do\n          {:event, %{ch: ?+}} -> model + 1\n          {:event, %{ch: ?-}} -> model - 1\n          _ -> model\n        end\n      end\n\n      def render(model) do\n        view do\n          label(content: \"Counter is #{model} (+/-)\")\n        end\n      end\n    end\n\n    Ratatouille.run(Counter)","ref":"Ratatouille.html#run/2-example"},{"type":"behaviour","title":"Ratatouille.App","doc":"Defines the `Ratatouille.App` behaviour. It provides the structure for\narchitecting both large and small terminal applications. This structure\nallows you to render views and update them over time or in response to user\ninput.","ref":"Ratatouille.App.html"},{"type":"behaviour","title":"A Simple Example - Ratatouille.App","doc":"defmodule Counter.App do\n      @behaviour Ratatouille.App\n\n      def model(_context), do: 0\n\n      def update(model, msg) do\n        case msg do\n          {:event, %{ch: ?+}} -> model + 1\n          {:event, %{ch: ?-}} -> model - 1\n          _ -> model\n        end\n      end\n\n      def render(model) do\n        view do\n          label(content: \"Counter is #{model} (+/-)\")\n        end\n      end\n    end","ref":"Ratatouille.App.html#module-a-simple-example"},{"type":"behaviour","title":"Architecture - Ratatouille.App","doc":"You may have recognized this pattern from [the Elm Architecture][1]. That's because\n`Ratatouille.App` is just a close translation of this architecture to Elixir.\nBecause Elixir and Elm are both functional programming languages, the pattern\nalso works very well in Elixir. It helps to centralize state,\n\nThe architecture cleanly separates logic into the following three parts:\n\n* **Model:** the state of your application\n* **Update:** a way to update your state\n* **View:** a way to view your state as a `%Ratatouille.Element{}` tree (think\n  HTML).\n\nThe behaviour callbacks map to these three parts:\n\n* `c:init/1` defines your initial model, using provided context if needed.\n* `c:update/2` handles a message and retuns a new model.\n* `c:render/1` receives a model and builds a view (an element tree) to view it.\n\nSee the documentation for each callback below for additional details.","ref":"Ratatouille.App.html#module-architecture"},{"type":"behaviour","title":"Runtime - Ratatouille.App","doc":"As long as you implement the behaviour callbacks, Ratatouille can handle the\nrest. It provides a runtime (`Ratatouille.Runtime`), which will handle\nactually running your application. That means setting up the window, rendering\nthe view, subscribing to events and passing these on to the application's\n`update/2` callback, and making sure the view is always re-rendered when the\napplication's model changes.","ref":"Ratatouille.App.html#module-runtime"},{"type":"behaviour","title":"Fetching Data & Other Expensive Calls - Ratatouille.App","doc":"A good rule of thumb for developing responsive UIs in any language is to never\nblock the UI thread. Here, we have a runtime process, but the same rule\napplies.\n\nIn Ratatouille, your application should use `Ratatouille.Runtime.Command` in\norder to start asynchronous tasks and get the result in the `update/2`\ncallback once finished.\n\n[1]: https://guide.elm-lang.org/architecture/","ref":"Ratatouille.App.html#module-fetching-data-other-expensive-calls"},{"type":"type","title":"Ratatouille.App.context/0","doc":"","ref":"Ratatouille.App.html#t:context/0"},{"type":"callback","title":"Ratatouille.App.init/1","doc":"The `init/1` callback defines the initial model. This model can be defined\nbased on the runtime context. See the \"Runtime Context\" section under\n`Ratatouille.Runtime` for details on what context is provided.","ref":"Ratatouille.App.html#c:init/1"},{"type":"type","title":"Ratatouille.App.model/0","doc":"","ref":"Ratatouille.App.html#t:model/0"},{"type":"type","title":"Ratatouille.App.msg/0","doc":"","ref":"Ratatouille.App.html#t:msg/0"},{"type":"callback","title":"Ratatouille.App.render/1","doc":"The `render/1` callback defines how to render the model as a view.\n\nIt should return a `%Ratatouille.Element{}` with the `:view` tag. For example:\n\n    @impl true\n    def render(model) do\n      view do\n        label(content: \"Hello, #{model.name}!\")\n      end\n    end","ref":"Ratatouille.App.html#c:render/1"},{"type":"callback","title":"Ratatouille.App.subscribe/1","doc":"The optional `subscribe/1` callback defines a subscription for the application\ngiven the initial model. Subscriptions are fulfilled via the `update/2`\ncallback.\n\nFor example, a subscription can be used to update the view every second (see\n`Ratatouille.Runtime.Subscription.interval/2`):\n\n    @impl true\n    def subscribe(_model) do\n      Subscription.interval(1_000, :tick)\n    end\n\nIt's also possible to subscribe to multiple things via\n`Ratatouille.Runtime.Subscription.batch/1`. See\n`Ratatouille.Runtime.Subscription` for more details.","ref":"Ratatouille.App.html#c:subscribe/1"},{"type":"callback","title":"Ratatouille.App.update/2","doc":"The `update/2` callback defines how to update the model in reaction to a\nmessage (for example, an event or a tick).\n\nThe following messages are currently passed to `update/2` by the runtime:\n\n* `{:event, event}` - A keyboard or click event.\n* `{:refresh, event}` - A resize event.\n\nThe callback should always return the model. It can be the same model or an\nupdated one. If the model changes, the runtime will know to re-render the\nmodel and update the window.","ref":"Ratatouille.App.html#c:update/2"},{"type":"module","title":"Ratatouille.Constants","doc":"A convenience wrapper of `ExTermbox.Constants`.","ref":"Ratatouille.Constants.html"},{"type":"function","title":"Ratatouille.Constants.attribute/1","doc":"See `ExTermbox.Constants.attribute/1`.","ref":"Ratatouille.Constants.html#attribute/1"},{"type":"function","title":"Ratatouille.Constants.attributes/0","doc":"See `ExTermbox.Constants.attributes/0`.","ref":"Ratatouille.Constants.html#attributes/0"},{"type":"function","title":"Ratatouille.Constants.color/1","doc":"See `ExTermbox.Constants.color/1`.","ref":"Ratatouille.Constants.html#color/1"},{"type":"function","title":"Ratatouille.Constants.colors/0","doc":"See `ExTermbox.Constants.colors/0`.","ref":"Ratatouille.Constants.html#colors/0"},{"type":"function","title":"Ratatouille.Constants.error_code/1","doc":"See `ExTermbox.Constants.error_code/1`.","ref":"Ratatouille.Constants.html#error_code/1"},{"type":"function","title":"Ratatouille.Constants.error_codes/0","doc":"See `ExTermbox.Constants.error_codes/0`.","ref":"Ratatouille.Constants.html#error_codes/0"},{"type":"function","title":"Ratatouille.Constants.event_type/1","doc":"See `ExTermbox.Constants.event_type/1`.","ref":"Ratatouille.Constants.html#event_type/1"},{"type":"function","title":"Ratatouille.Constants.event_types/0","doc":"See `ExTermbox.Constants.event_types/0`.","ref":"Ratatouille.Constants.html#event_types/0"},{"type":"function","title":"Ratatouille.Constants.input_mode/1","doc":"See `ExTermbox.Constants.input_mode/1`.","ref":"Ratatouille.Constants.html#input_mode/1"},{"type":"function","title":"Ratatouille.Constants.input_modes/0","doc":"See `ExTermbox.Constants.input_modes/0`.","ref":"Ratatouille.Constants.html#input_modes/0"},{"type":"function","title":"Ratatouille.Constants.key/1","doc":"See `ExTermbox.Constants.key/1`.","ref":"Ratatouille.Constants.html#key/1"},{"type":"function","title":"Ratatouille.Constants.keys/0","doc":"See `ExTermbox.Constants.keys/0`.","ref":"Ratatouille.Constants.html#keys/0"},{"type":"function","title":"Ratatouille.Constants.output_mode/1","doc":"See `ExTermbox.Constants.output_mode/1`.","ref":"Ratatouille.Constants.html#output_mode/1"},{"type":"function","title":"Ratatouille.Constants.output_modes/0","doc":"See `ExTermbox.Constants.output_modes/0`.","ref":"Ratatouille.Constants.html#output_modes/0"},{"type":"module","title":"Ratatouille.EventManager","doc":"A wrapper of `ExTermbox.EventManager` so that Ratatouille applications don't\nneed to use or depend on ex_termbox directly.","ref":"Ratatouille.EventManager.html"},{"type":"function","title":"Ratatouille.EventManager.child_spec/1","doc":"Provides a child specification to use when starting the event manager under a\nsupervisor.","ref":"Ratatouille.EventManager.html#child_spec/1"},{"type":"function","title":"Ratatouille.EventManager.start_link/1","doc":"Starts the `ExTermbox.EventManager` gen_server.","ref":"Ratatouille.EventManager.html#start_link/1"},{"type":"function","title":"Ratatouille.EventManager.stop/1","doc":"Stops the event manager server.","ref":"Ratatouille.EventManager.html#stop/1"},{"type":"function","title":"Ratatouille.EventManager.subscribe/2","doc":"Subscribes the given pid to future event notifications.","ref":"Ratatouille.EventManager.html#subscribe/2"},{"type":"behaviour","title":"Ratatouille.Renderer","doc":"Logic to render a view tree.\n\nThis API is still under development.","ref":"Ratatouille.Renderer.html"},{"type":"type","title":"Ratatouille.Renderer.child_element/0","doc":"","ref":"Ratatouille.Renderer.html#t:child_element/0"},{"type":"type","title":"Ratatouille.Renderer.child_tag/0","doc":"","ref":"Ratatouille.Renderer.html#t:child_tag/0"},{"type":"function","title":"Ratatouille.Renderer.render/2","doc":"Renders a view tree to canvas, given a canvas and a root element (an element\nwith the `:view` tag).\n\nThe tree is rendered by recursively rendering each element in the hierarchy.\nThe canvas serves as both the accumulator for rendered cells at each stage and\nas the box representing available space for rendering, which shrinks as this\nspace is consumed.","ref":"Ratatouille.Renderer.html#render/2"},{"type":"callback","title":"Ratatouille.Renderer.render/3","doc":"","ref":"Ratatouille.Renderer.html#c:render/3"},{"type":"function","title":"Ratatouille.Renderer.render_tree/2","doc":"","ref":"Ratatouille.Renderer.html#render_tree/2"},{"type":"type","title":"Ratatouille.Renderer.root_element/0","doc":"","ref":"Ratatouille.Renderer.html#t:root_element/0"},{"type":"function","title":"Ratatouille.Renderer.validate_tree/1","doc":"Validates the hierarchy of a view tree given the root element.\n\nUsed by the render/2 function to prevent strange errors that may otherwise\noccur when processing invalid view trees.","ref":"Ratatouille.Renderer.html#validate_tree/1"},{"type":"module","title":"Ratatouille.Renderer.Attributes","doc":"Functions for working with element attributes","ref":"Ratatouille.Renderer.Attributes.html"},{"type":"function","title":"Ratatouille.Renderer.Attributes.to_terminal_attribute/1","doc":"","ref":"Ratatouille.Renderer.Attributes.html#to_terminal_attribute/1"},{"type":"function","title":"Ratatouille.Renderer.Attributes.to_terminal_color/1","doc":"","ref":"Ratatouille.Renderer.Attributes.html#to_terminal_color/1"},{"type":"module","title":"Ratatouille.Renderer.Border","doc":"Primitives for rendering borders","ref":"Ratatouille.Renderer.Border.html"},{"type":"function","title":"Ratatouille.Renderer.Border.render/2","doc":"","ref":"Ratatouille.Renderer.Border.html#render/2"},{"type":"module","title":"Ratatouille.Renderer.Box","doc":"This defines the internal representation of a rectangular region---a box---for\nrendering, as well as logic for transforming these boxes.\n\nBoxes live on a coordinate plane. The y-axis is inverted so that the y values\nincrease as the box's height grows.\n\n    --------------> x\n    |\n    |    ________\n    |    |      |\n    |    |______|\n    v\n    y\n\nA `Box` struct stores the coordinates for two corners of the box---the\ntop-left and bottom-right corners--from which the remaining attributes\n(height, width, other corners) can be computed.\n\n     _________________\n    |                 |\n    |  A              |\n    |                 |\n    |                 |\n    |                 |\n    |              B  |\n    |_________________|\n\n    A: top-left corner, e.g. (0, 0)\n    B: bottom-right corner, e.g. (10, 10)\n\nFor rendering purposes, the outermost box will typically have a top-left\ncorner (0, 0) and a bottom-right corner (x, y) where x is the number of rows\nand y is the number of columns on the terminal.\n\nThis outermost box can then be subdivided as necessary to render different\nelements of the view.","ref":"Ratatouille.Renderer.Box.html"},{"type":"function","title":"Ratatouille.Renderer.Box.bottom_left/1","doc":"","ref":"Ratatouille.Renderer.Box.html#bottom_left/1"},{"type":"function","title":"Ratatouille.Renderer.Box.bottom_right/1","doc":"","ref":"Ratatouille.Renderer.Box.html#bottom_right/1"},{"type":"function","title":"Ratatouille.Renderer.Box.consume/3","doc":"","ref":"Ratatouille.Renderer.Box.html#consume/3"},{"type":"function","title":"Ratatouille.Renderer.Box.contains?/2","doc":"","ref":"Ratatouille.Renderer.Box.html#contains?/2"},{"type":"function","title":"Ratatouille.Renderer.Box.from_dimensions/3","doc":"","ref":"Ratatouille.Renderer.Box.html#from_dimensions/3"},{"type":"function","title":"Ratatouille.Renderer.Box.head/2","doc":"Given a box, returns a slice of the y axis with `n` rows from the top.","ref":"Ratatouille.Renderer.Box.html#head/2"},{"type":"function","title":"Ratatouille.Renderer.Box.height/1","doc":"","ref":"Ratatouille.Renderer.Box.html#height/1"},{"type":"function","title":"Ratatouille.Renderer.Box.padded/2","doc":"","ref":"Ratatouille.Renderer.Box.html#padded/2"},{"type":"function","title":"Ratatouille.Renderer.Box.positions/1","doc":"","ref":"Ratatouille.Renderer.Box.html#positions/1"},{"type":"function","title":"Ratatouille.Renderer.Box.tail/2","doc":"Given a box, returns a slice of the y axis with `n` rows from the bottom.","ref":"Ratatouille.Renderer.Box.html#tail/2"},{"type":"function","title":"Ratatouille.Renderer.Box.top_left/1","doc":"","ref":"Ratatouille.Renderer.Box.html#top_left/1"},{"type":"function","title":"Ratatouille.Renderer.Box.top_right/1","doc":"","ref":"Ratatouille.Renderer.Box.html#top_right/1"},{"type":"function","title":"Ratatouille.Renderer.Box.translate/3","doc":"","ref":"Ratatouille.Renderer.Box.html#translate/3"},{"type":"function","title":"Ratatouille.Renderer.Box.width/1","doc":"","ref":"Ratatouille.Renderer.Box.html#width/1"},{"type":"module","title":"Ratatouille.Renderer.Canvas","doc":"A canvas represents a terminal window, a subvision of it for rendering, and a\nsparse mapping of positions to cells.\n\nA `%Canvas{}` struct can be rendered to different output formats. This includes\nthe primary use-case of rendering to the termbox-managed window, but also\nrendering to strings, which is useful for testing.","ref":"Ratatouille.Renderer.Canvas.html"},{"type":"function","title":"Ratatouille.Renderer.Canvas.consume/3","doc":"Copies the canvas to a new one with the render box consumed by the given `dx`\nand `dy`.\n\nThe render box is used to indicate the empty, renderable space on the canvas,\nso this might be called with a `dy` of 1 after rendering a line of text. The\nbox is consumed left-to-right and top-to-bottom.","ref":"Ratatouille.Renderer.Canvas.html#consume/3"},{"type":"function","title":"Ratatouille.Renderer.Canvas.consume_columns/2","doc":"Creates a new canvas with `n` columns (from the left) consumed.","ref":"Ratatouille.Renderer.Canvas.html#consume_columns/2"},{"type":"function","title":"Ratatouille.Renderer.Canvas.consume_rows/2","doc":"Creates a new canvas with `n` rows (from the top) consumed.","ref":"Ratatouille.Renderer.Canvas.html#consume_rows/2"},{"type":"function","title":"Ratatouille.Renderer.Canvas.fill_background/1","doc":"","ref":"Ratatouille.Renderer.Canvas.html#fill_background/1"},{"type":"function","title":"Ratatouille.Renderer.Canvas.from_dimensions/2","doc":"Creates an empty canvas with the given dimensions.","ref":"Ratatouille.Renderer.Canvas.html#from_dimensions/2"},{"type":"function","title":"Examples - Ratatouille.Renderer.Canvas.from_dimensions/2","doc":"iex> Canvas.from_dimensions(10, 20)\n    %Canvas{\n      outer_box: %Ratatouille.Renderer.Box{\n        top_left: %ExTermbox.Position{x: 0, y: 0},\n        bottom_right: %ExTermbox.Position{x: 9, y: 19}\n      },\n      render_box: %Ratatouille.Renderer.Box{\n        top_left: %ExTermbox.Position{x: 0, y: 0},\n        bottom_right: %ExTermbox.Position{x: 9, y: 19}\n      },\n      cells: %{}\n    }","ref":"Ratatouille.Renderer.Canvas.html#from_dimensions/2-examples"},{"type":"function","title":"Ratatouille.Renderer.Canvas.merge_cells/2","doc":"Merges a list of cells into the canvas, provided that the cells are located\nwithin the canvas's rendering box. Returns a new canvas with the merged cells.","ref":"Ratatouille.Renderer.Canvas.html#merge_cells/2"},{"type":"function","title":"Ratatouille.Renderer.Canvas.padded/2","doc":"Copies the canvas to a new one with the render box padded on each side (top,\nleft, bottom, right) by `size`. Pass a negative size to remove padding.","ref":"Ratatouille.Renderer.Canvas.html#padded/2"},{"type":"function","title":"Ratatouille.Renderer.Canvas.put_box/2","doc":"","ref":"Ratatouille.Renderer.Canvas.html#put_box/2"},{"type":"function","title":"Ratatouille.Renderer.Canvas.render_to_string/1","doc":"","ref":"Ratatouille.Renderer.Canvas.html#render_to_string/1"},{"type":"function","title":"Ratatouille.Renderer.Canvas.render_to_strings/1","doc":"","ref":"Ratatouille.Renderer.Canvas.html#render_to_strings/1"},{"type":"function","title":"Ratatouille.Renderer.Canvas.render_to_termbox/2","doc":"","ref":"Ratatouille.Renderer.Canvas.html#render_to_termbox/2"},{"type":"type","title":"Ratatouille.Renderer.Canvas.t/0","doc":"","ref":"Ratatouille.Renderer.Canvas.html#t:t/0"},{"type":"function","title":"Ratatouille.Renderer.Canvas.translate/3","doc":"","ref":"Ratatouille.Renderer.Canvas.html#translate/3"},{"type":"module","title":"Ratatouille.Renderer.Cells","doc":"Functions for working with canvas cells.","ref":"Ratatouille.Renderer.Cells.html"},{"type":"function","title":"Ratatouille.Renderer.Cells.background/1","doc":"Computes a cell's background given a standardized attributes map.","ref":"Ratatouille.Renderer.Cells.html#background/1"},{"type":"function","title":"Ratatouille.Renderer.Cells.foreground/1","doc":"Computes a cell's foreground given a standardized attributes map.\n\nThe foreground value is computed by taking the color's integer value and the\ninteger values of any styling attributes (e.g., bold, underline) and computing\nthe bitwise OR of all the values.","ref":"Ratatouille.Renderer.Cells.html#foreground/1"},{"type":"function","title":"Ratatouille.Renderer.Cells.generator/3","doc":"Given a starting position, orientation and cell template, returns a cell\ngenerator which can be used to iteratively generate a row or column of cells.","ref":"Ratatouille.Renderer.Cells.html#generator/3"},{"type":"module","title":"Ratatouille.Renderer.Line","doc":"Primitives for rendering lines","ref":"Ratatouille.Renderer.Line.html"},{"type":"function","title":"Ratatouille.Renderer.Line.render/6","doc":"","ref":"Ratatouille.Renderer.Line.html#render/6"},{"type":"function","title":"Ratatouille.Renderer.Line.render_horizontal/5","doc":"","ref":"Ratatouille.Renderer.Line.html#render_horizontal/5"},{"type":"function","title":"Ratatouille.Renderer.Line.render_vertical/5","doc":"","ref":"Ratatouille.Renderer.Line.html#render_vertical/5"},{"type":"module","title":"Ratatouille.Renderer.Text","doc":"Primitives for rendering text","ref":"Ratatouille.Renderer.Text.html"},{"type":"function","title":"Ratatouille.Renderer.Text.render/4","doc":"","ref":"Ratatouille.Renderer.Text.html#render/4"},{"type":"function","title":"Ratatouille.Renderer.Text.render_group/3","doc":"","ref":"Ratatouille.Renderer.Text.html#render_group/3"},{"type":"module","title":"Ratatouille.Runtime","doc":"A runtime for apps implementing the `Ratatouille.App` behaviour. See\n`Ratatouille.App` for details on how to build apps.","ref":"Ratatouille.Runtime.html"},{"type":"module","title":"Runtime Context - Ratatouille.Runtime","doc":"The runtime provides a map with additional context to the app's\n`c:Ratatouille.App.init/1` callback. This can be used, for example,\nto get information about the terminal window.  Currently, the following\nattributes are provided via the map:\n\n* `:window`: A map with `:height` and `:width` keys.","ref":"Ratatouille.Runtime.html#module-runtime-context"},{"type":"module","title":"Shutdown Options - Ratatouille.Runtime","doc":"When a quit event is received (by default, \"q\", \"Q\" or ctrl-c), the runtime\nwill carry out the configured shutdown strategy. The default strategy\n(`:self`) just stops the runtime process itself. In certain cases, it may\nbe desirable to stop an OTP application or the system itself, so a few other\nstrategies are provided:\n\n* `:supervisor` - Stops the runtime supervisor, which will terminate the\n  runtime process and other dependencies. The VM will still be running\n  afterwards unless stopped by other means.\n* `{:application, name}` - Stops the named application. The VM will still be\n  running afterwards unless stopped by other means.\n* `:system`: Gracefully stops the system (calls `:init.stop/0`). Gracefully\n  shutting down the VM ensures that all applications are stopped. There will\n  be some lag between sending the quit event and the OS process actually\n  exiting, as the VM needs about a second to stop this way. See the note below\n  if this is a problem.\n\nIf the `:system` option is too slow, it's possible to halt the system manually\nwith `System.halt/0`. It's just important to ensure that the system is only\nhalted after the terminal was restored to its original state. Otherwise, there\nmay be rendering artifacts or issues with the cursor. The `:supervisor` option\ncan be used to properly halt the system by monitoring the runtime supervisor\nand halting once it has exited. Alternatively, use the `:application` option\nto stop an application that runs the runtime supervisor and halt in the\napplication's `stop/1` callback.","ref":"Ratatouille.Runtime.html#module-shutdown-options"},{"type":"function","title":"Ratatouille.Runtime.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\n`arg` is passed as the argument to `Task.start_link/1` in the `:start` field\nof the spec.\n\nFor more information, see the `Supervisor` module,\nthe `Supervisor.child_spec/2` function and the `t:Supervisor.child_spec/0` type.","ref":"Ratatouille.Runtime.html#child_spec/1"},{"type":"function","title":"Ratatouille.Runtime.run/1","doc":"","ref":"Ratatouille.Runtime.html#run/1"},{"type":"function","title":"Ratatouille.Runtime.start_link/1","doc":"Starts the application runtime given a module defining a Ratatouille terminal\napplication.","ref":"Ratatouille.Runtime.html#start_link/1"},{"type":"function","title":"Configuration - Ratatouille.Runtime.start_link/1","doc":"* `:app` (required) - The `Ratatouille.App` to run.\n* `:shutdown` - The strategy for stopping the terminal application when a quit\n   event is received.\n* `:interval` - The runtime loop interval in milliseconds. The default\n   interval is 500 ms. A subscription can be fulfilled at most once every\n   interval, so this effectively configures the maximum subscription\n   resolution that's possible.\n* `:quit_events` - Specifies the events that should quit the terminal\n   application. Given as a list of tuples conforming where each tuple conforms\n   to either `{:ch, ch}` or `{:key, key}`. If not specified, ctrl-c and q / Q\n   can be used to quit the application by default.","ref":"Ratatouille.Runtime.html#start_link/1-configuration"},{"type":"module","title":"Ratatouille.Runtime.Command","doc":"Commands provide a way to start an expensive call in the background and get\nthe result back via `c:Ratatouille.App.update/2`.\n\nCommands should be constructed via the functions below and not via the struct\ndirectly, as this is internal and subject to change.","ref":"Ratatouille.Runtime.Command.html"},{"type":"function","title":"Ratatouille.Runtime.Command.batch/1","doc":"Returns a batch command given a list of commands. This simply provides a way\nto return multiple commands as a single one. Batch commands should not depend\non one another---Ratatouille's runtime may run some or all of them in\nparallel and doesn't guarantee any particular order of execution.\n\nDependencies should be expressed via a single command or a sequence of\ncommands orchestrated via the application model state.","ref":"Ratatouille.Runtime.Command.html#batch/1"},{"type":"function","title":"Ratatouille.Runtime.Command.new/2","doc":"Returns a new command that can be returned in the `c:Ratatouille.App.update/2`\nor `c:Ratatouille.App.init/1` callbacks.\n\nTakes an anonymous function and a message. The message is used to send a\nresponse back to your app along with the result. It can be any Erlang term, so\nit's also possible to include identifiers (e.g., `{:finished, id}`).","ref":"Ratatouille.Runtime.Command.html#new/2"},{"type":"opaque","title":"Ratatouille.Runtime.Command.t/0","doc":"","ref":"Ratatouille.Runtime.Command.html#t:t/0"},{"type":"module","title":"Ratatouille.Runtime.State","doc":"Defines a struct to store the runtime loop's state.","ref":"Ratatouille.Runtime.State.html"},{"type":"module","title":"Ratatouille.Runtime.Subscription","doc":"Subscriptions provide a way for the app to be notified via\n`c:Ratatouille.App.update/2` when something interesting happens.\n\nSubscriptions should be constructed via the functions below and not via the\nstruct directly, as this is internal and subject to change.\n\nCurrently, it's possible to subscribe to time intervals (`interval/2`) and to\ncreate batch subscriptions (i.e., multiple time intervals). More subscription\ntypes may be introduced later.","ref":"Ratatouille.Runtime.Subscription.html"},{"type":"module","title":"Accuracy of Time Intervals - Ratatouille.Runtime.Subscription","doc":"Ratatouille's runtime loop, which handles subscriptions, runs on a interval\nitself (by default, every 500 ms). This means that the runtime loop is the\nminimum possible subscription interval. If a subscription's interval is more\nfrequent than the runtime loop interval, the runtime loop interval is the\nsubscription's effective interval.\n\nThere is also no guarantee that subscriptions will be processed on time, as\nthe runtime may be busy with other tasks (e.g., handling events or rendering).\nWith that said, if care is taken to keep expensive calls out of the runtime\nloop, subscriptions should be processed very close to requested interval.","ref":"Ratatouille.Runtime.Subscription.html#module-accuracy-of-time-intervals"},{"type":"function","title":"Ratatouille.Runtime.Subscription.batch/1","doc":"Creates a batch subscription from a list of subscriptions.\n\nThis provides a way to subscribe to multiple things, while still returning a\nsingle subscription in `c:Ratatouille.App.subscribe/1`.","ref":"Ratatouille.Runtime.Subscription.html#batch/1"},{"type":"function","title":"Ratatouille.Runtime.Subscription.interval/2","doc":"Returns a subscription based on a time interval. Takes the number of\nmilliseconds (`ms`) and a message as arguments. When returned in the\n`c:Ratatouille.App.subscribe/1` callback, the runtime will call the\n`c:Ratatouille.App.update/2` function with current model and the message, on\napproximately the given interval. See above for details on what\n\"approximately\" means here.","ref":"Ratatouille.Runtime.Subscription.html#interval/2"},{"type":"opaque","title":"Ratatouille.Runtime.Subscription.t/0","doc":"","ref":"Ratatouille.Runtime.Subscription.html#t:t/0"},{"type":"module","title":"Ratatouille.Runtime.Supervisor","doc":"A supervisor to run the application runtime and its dependencies.","ref":"Ratatouille.Runtime.Supervisor.html"},{"type":"function","title":"Ratatouille.Runtime.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Ratatouille.Runtime.Supervisor.html#child_spec/1"},{"type":"function","title":"Ratatouille.Runtime.Supervisor.start_link/1","doc":"Starts a supervisor that manages `Ratatouille.Runtime`, along with runtime\ndependencies `Ratatouille.EventManager` and `Ratatouille.Window`.","ref":"Ratatouille.Runtime.Supervisor.html#start_link/1"},{"type":"function","title":"Options - Ratatouille.Runtime.Supervisor.start_link/1","doc":"* `:runtime` - Options for the runtime. See `Ratatouille.Runtime.start_link/1`.","ref":"Ratatouille.Runtime.Supervisor.html#start_link/1-options"},{"type":"module","title":"Ratatouille.View","doc":"In Ratatouille, a view is simply a tree of elements. Each element in the tree\nholds an attributes map and a list of zero or more child nodes. Visually, it\nlooks like something this:\n\n    %Element{\n      tag: :view,\n      attributes: %{},\n      children: [\n        %Element{\n          tag: :row,\n          attributes: %{},\n          children: [\n            %Element{tag: :column, attributes: %{size: 4}, children: []},\n            %Element{tag: :column, attributes: %{size: 4}, children: []},\n            %Element{tag: :column, attributes: %{size: 4}, children: []}\n          ]\n        }\n      ]\n    }","ref":"Ratatouille.View.html"},{"type":"module","title":"View DSL - Ratatouille.View","doc":"Because it's a bit tedious to define views like that, Ratatouille provides a\nDSL to define them without all the boilerplate.\n\nNow we can turn the above into this:\n\n    view do\n      row do\n        column(size: 4)\n        column(size: 4)\n        column(size: 4)\n      end\n    end\n\nWhile the syntax is more compact, the end result is exactly the same. This\nexpression produces the exact same `%Element{}` struct as defined above.\n\nTo use the DSL like this, we need to import all the functions:\n\n    import Ratatouille.View\n\nAlternatively, import just the ones you need:\n\n    import Ratatouille.View, only: [view: 0, row: 0, column: 1]","ref":"Ratatouille.View.html#module-view-dsl"},{"type":"module","title":"Forms - Ratatouille.View","doc":"All of the possible forms are enumerated below.\n\nElement with tag `foo`:\n\n    foo()\n\nElement with tag `foo` and attributes:\n\n    foo(size: 42)\n\nElement with tag `foo` and children as list:\n\n    foo([\n      bar()\n    ])\n\nElement with tag `foo` and children as block:\n\n    foo do\n      bar()\n    end\n\nElement with tag `foo`, attributes, and children as list:\n\n    foo(\n      [size: 42],\n      [bar()]\n    )\n\nElement with tag `foo`, attributes, and children as block:\n\n    foo size: 42 do\n      bar()\n    end","ref":"Ratatouille.View.html#module-forms"},{"type":"module","title":"Empty Elements - Ratatouille.View","doc":"Similar to so-called \"empty\" HTML elements such as ` `, Ratatouille also\nhas elements for which passing content doesn't make sense. For example, the\nleaf node `text` stores its content in its attributes and cannot have any\nchild elements of its own.\n\nIn such cases, the block and list forms are unsupported.","ref":"Ratatouille.View.html#module-empty-elements"},{"type":"module","title":"Validation - Ratatouille.View","doc":"While some errors---such as passing children to empty elements---are prevented\nby the DSL, it's still possible (for now, at least) to build\nsemantically-invalid element trees using the DSL. This means that the elements\nare being used in a way that doesn't make sense to the renderer.\n\nIn order to prevent cryptic rendering errors, the renderer first validates the\nelement tree it's given and rejects the whole thing if the structure is\nunsupported. It currently checks the following things:\n\n* The top-level element passed to the renderer must have the `:view` tag.\n* A parent element may only have child elements that have one of the\n  supported child tags for the parent element.\n* An element must define all of its required attributes and may not define any\n  unknown attributes.\n\nThe last two rules are based on the element's specification in\n`Ratatouille.Renderer.Element`.","ref":"Ratatouille.View.html#module-validation"},{"type":"macro","title":"Ratatouille.View.bar/0","doc":"Defines an element with the `:bar` tag.","ref":"Ratatouille.View.html#bar/0"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.bar/0","doc":"* label","ref":"Ratatouille.View.html#bar/0-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.bar/0","doc":"Empty element:\n\n    bar()\n\nWith a block:\n\n    bar do\n      # ...child elements...\n    end","ref":"Ratatouille.View.html#bar/0-examples"},{"type":"macro","title":"Ratatouille.View.bar/1","doc":"Defines an element with the `:bar` tag and either:\n\n* given attributes and an optional block\n* a list of child elements","ref":"Ratatouille.View.html#bar/1"},{"type":"macro","title":"Attributes - Ratatouille.View.bar/1","doc":"None","ref":"Ratatouille.View.html#bar/1-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.bar/1","doc":"* label","ref":"Ratatouille.View.html#bar/1-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.bar/1","doc":"Passing attributes:\n\n    bar(key: value)\n\nPassing attributes and a block:\n\n    bar(key: value) do\n      # ...child elements...\n    end\n\nPassing list of children:\n\n    bar([elem1, elem2])","ref":"Ratatouille.View.html#bar/1-examples"},{"type":"macro","title":"Ratatouille.View.bar/2","doc":"Defines an element with the `:bar` tag and the given attributes and\nchild elements.","ref":"Ratatouille.View.html#bar/2"},{"type":"macro","title":"Attributes - Ratatouille.View.bar/2","doc":"None","ref":"Ratatouille.View.html#bar/2-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.bar/2","doc":"* label","ref":"Ratatouille.View.html#bar/2-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.bar/2","doc":"bar([key: value], [elem1, elem2])","ref":"Ratatouille.View.html#bar/2-examples"},{"type":"macro","title":"Ratatouille.View.canvas/0","doc":"Defines an element with the `:canvas` tag.","ref":"Ratatouille.View.html#canvas/0"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.canvas/0","doc":"* canvas_cell","ref":"Ratatouille.View.html#canvas/0-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.canvas/0","doc":"Empty element:\n\n    canvas()\n\nWith a block:\n\n    canvas do\n      # ...child elements...\n    end","ref":"Ratatouille.View.html#canvas/0-examples"},{"type":"macro","title":"Ratatouille.View.canvas/1","doc":"Defines an element with the `:canvas` tag and either:\n\n* given attributes and an optional block\n* a list of child elements","ref":"Ratatouille.View.html#canvas/1"},{"type":"macro","title":"Attributes - Ratatouille.View.canvas/1","doc":"* `height` (required) - Integer representing the canvas height\n* `width` (required) - Integer representing the canvas width","ref":"Ratatouille.View.html#canvas/1-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.canvas/1","doc":"* canvas_cell","ref":"Ratatouille.View.html#canvas/1-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.canvas/1","doc":"Passing attributes:\n\n    canvas(key: value)\n\nPassing attributes and a block:\n\n    canvas(key: value) do\n      # ...child elements...\n    end\n\nPassing list of children:\n\n    canvas([elem1, elem2])","ref":"Ratatouille.View.html#canvas/1-examples"},{"type":"macro","title":"Ratatouille.View.canvas/2","doc":"Defines an element with the `:canvas` tag and the given attributes and\nchild elements.","ref":"Ratatouille.View.html#canvas/2"},{"type":"macro","title":"Attributes - Ratatouille.View.canvas/2","doc":"* `height` (required) - Integer representing the canvas height\n* `width` (required) - Integer representing the canvas width","ref":"Ratatouille.View.html#canvas/2-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.canvas/2","doc":"* canvas_cell","ref":"Ratatouille.View.html#canvas/2-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.canvas/2","doc":"canvas([key: value], [elem1, elem2])","ref":"Ratatouille.View.html#canvas/2-examples"},{"type":"macro","title":"Ratatouille.View.canvas_cell/0","doc":"Defines an element with the `:canvas_cell` tag.","ref":"Ratatouille.View.html#canvas_cell/0"},{"type":"macro","title":"Examples - Ratatouille.View.canvas_cell/0","doc":"Empty element:\n\n    canvas_cell()","ref":"Ratatouille.View.html#canvas_cell/0-examples"},{"type":"macro","title":"Ratatouille.View.canvas_cell/1","doc":"Defines an element with the `:canvas_cell` tag and the given attributes.","ref":"Ratatouille.View.html#canvas_cell/1"},{"type":"macro","title":"Attributes - Ratatouille.View.canvas_cell/1","doc":"* `x` (required) - Integer representing the cell's column (zero-indexed)\n* `y` (required) - Integer representing the cell's row (zero-indexed)\n* `color` (optional) - Constant representing color to use for foreground\n* `char` (optional) - Single character to render within this cell\n* `background` (optional) - Constant representing color to use for background\n* `attributes` (optional) - Constant representing style attributes to apply","ref":"Ratatouille.View.html#canvas_cell/1-attributes"},{"type":"macro","title":"Examples - Ratatouille.View.canvas_cell/1","doc":"canvas_cell(key: value)","ref":"Ratatouille.View.html#canvas_cell/1-examples"},{"type":"macro","title":"Ratatouille.View.chart/0","doc":"Defines an element with the `:chart` tag.","ref":"Ratatouille.View.html#chart/0"},{"type":"macro","title":"Examples - Ratatouille.View.chart/0","doc":"Empty element:\n\n    chart()","ref":"Ratatouille.View.html#chart/0-examples"},{"type":"macro","title":"Ratatouille.View.chart/1","doc":"Defines an element with the `:chart` tag and the given attributes.","ref":"Ratatouille.View.html#chart/1"},{"type":"macro","title":"Attributes - Ratatouille.View.chart/1","doc":"* `series` (required) - List of float or integer values representing the series\n* `type` (required) - Type of chart to plot. Currently only `:line` is supported\n* `height` (optional) - Height of the chart in rows","ref":"Ratatouille.View.html#chart/1-attributes"},{"type":"macro","title":"Examples - Ratatouille.View.chart/1","doc":"chart(key: value)","ref":"Ratatouille.View.html#chart/1-examples"},{"type":"macro","title":"Ratatouille.View.column/0","doc":"Defines an element with the `:column` tag.","ref":"Ratatouille.View.html#column/0"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.column/0","doc":"* canvas\n* chart\n* label\n* panel\n* row\n* sparkline\n* table\n* tree\n* viewport\n* progress_bar","ref":"Ratatouille.View.html#column/0-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.column/0","doc":"Empty element:\n\n    column()\n\nWith a block:\n\n    column do\n      # ...child elements...\n    end","ref":"Ratatouille.View.html#column/0-examples"},{"type":"macro","title":"Ratatouille.View.column/1","doc":"Defines an element with the `:column` tag and either:\n\n* given attributes and an optional block\n* a list of child elements","ref":"Ratatouille.View.html#column/1"},{"type":"macro","title":"Attributes - Ratatouille.View.column/1","doc":"* `size` (required) - Number of units on the grid that the column should occupy","ref":"Ratatouille.View.html#column/1-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.column/1","doc":"* canvas\n* chart\n* label\n* panel\n* row\n* sparkline\n* table\n* tree\n* viewport\n* progress_bar","ref":"Ratatouille.View.html#column/1-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.column/1","doc":"Passing attributes:\n\n    column(key: value)\n\nPassing attributes and a block:\n\n    column(key: value) do\n      # ...child elements...\n    end\n\nPassing list of children:\n\n    column([elem1, elem2])","ref":"Ratatouille.View.html#column/1-examples"},{"type":"macro","title":"Ratatouille.View.column/2","doc":"Defines an element with the `:column` tag and the given attributes and\nchild elements.","ref":"Ratatouille.View.html#column/2"},{"type":"macro","title":"Attributes - Ratatouille.View.column/2","doc":"* `size` (required) - Number of units on the grid that the column should occupy","ref":"Ratatouille.View.html#column/2-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.column/2","doc":"* canvas\n* chart\n* label\n* panel\n* row\n* sparkline\n* table\n* tree\n* viewport\n* progress_bar","ref":"Ratatouille.View.html#column/2-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.column/2","doc":"column([key: value], [elem1, elem2])","ref":"Ratatouille.View.html#column/2-examples"},{"type":"function","title":"Ratatouille.View.element/2","doc":"","ref":"Ratatouille.View.html#element/2"},{"type":"function","title":"Ratatouille.View.element/3","doc":"","ref":"Ratatouille.View.html#element/3"},{"type":"macro","title":"Ratatouille.View.label/0","doc":"Defines an element with the `:label` tag.","ref":"Ratatouille.View.html#label/0"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.label/0","doc":"* text","ref":"Ratatouille.View.html#label/0-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.label/0","doc":"Empty element:\n\n    label()\n\nWith a block:\n\n    label do\n      # ...child elements...\n    end","ref":"Ratatouille.View.html#label/0-examples"},{"type":"macro","title":"Ratatouille.View.label/1","doc":"Defines an element with the `:label` tag and either:\n\n* given attributes and an optional block\n* a list of child elements","ref":"Ratatouille.View.html#label/1"},{"type":"macro","title":"Attributes - Ratatouille.View.label/1","doc":"* `content` (optional) - Binary containing the text content to be displayed\n* `color` (optional) - Constant representing color to use for foreground\n* `background` (optional) - Constant representing color to use for background\n* `attributes` (optional) - Constant representing style attributes to apply\n* `wrap` (optional) - Boolean indicating whether or not to wrap lines to fit available space","ref":"Ratatouille.View.html#label/1-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.label/1","doc":"* text","ref":"Ratatouille.View.html#label/1-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.label/1","doc":"Passing attributes:\n\n    label(key: value)\n\nPassing attributes and a block:\n\n    label(key: value) do\n      # ...child elements...\n    end\n\nPassing list of children:\n\n    label([elem1, elem2])","ref":"Ratatouille.View.html#label/1-examples"},{"type":"macro","title":"Ratatouille.View.label/2","doc":"Defines an element with the `:label` tag and the given attributes and\nchild elements.","ref":"Ratatouille.View.html#label/2"},{"type":"macro","title":"Attributes - Ratatouille.View.label/2","doc":"* `content` (optional) - Binary containing the text content to be displayed\n* `color` (optional) - Constant representing color to use for foreground\n* `background` (optional) - Constant representing color to use for background\n* `attributes` (optional) - Constant representing style attributes to apply\n* `wrap` (optional) - Boolean indicating whether or not to wrap lines to fit available space","ref":"Ratatouille.View.html#label/2-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.label/2","doc":"* text","ref":"Ratatouille.View.html#label/2-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.label/2","doc":"label([key: value], [elem1, elem2])","ref":"Ratatouille.View.html#label/2-examples"},{"type":"macro","title":"Ratatouille.View.overlay/0","doc":"Defines an element with the `:overlay` tag.","ref":"Ratatouille.View.html#overlay/0"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.overlay/0","doc":"* canvas\n* chart\n* label\n* panel\n* row\n* sparkline\n* table\n* tree\n* viewport\n* progress_bar","ref":"Ratatouille.View.html#overlay/0-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.overlay/0","doc":"Empty element:\n\n    overlay()\n\nWith a block:\n\n    overlay do\n      # ...child elements...\n    end","ref":"Ratatouille.View.html#overlay/0-examples"},{"type":"macro","title":"Ratatouille.View.overlay/1","doc":"Defines an element with the `:overlay` tag and either:\n\n* given attributes and an optional block\n* a list of child elements","ref":"Ratatouille.View.html#overlay/1"},{"type":"macro","title":"Attributes - Ratatouille.View.overlay/1","doc":"* `padding` (optional) - Integer number of units of padding","ref":"Ratatouille.View.html#overlay/1-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.overlay/1","doc":"* canvas\n* chart\n* label\n* panel\n* row\n* sparkline\n* table\n* tree\n* viewport\n* progress_bar","ref":"Ratatouille.View.html#overlay/1-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.overlay/1","doc":"Passing attributes:\n\n    overlay(key: value)\n\nPassing attributes and a block:\n\n    overlay(key: value) do\n      # ...child elements...\n    end\n\nPassing list of children:\n\n    overlay([elem1, elem2])","ref":"Ratatouille.View.html#overlay/1-examples"},{"type":"macro","title":"Ratatouille.View.overlay/2","doc":"Defines an element with the `:overlay` tag and the given attributes and\nchild elements.","ref":"Ratatouille.View.html#overlay/2"},{"type":"macro","title":"Attributes - Ratatouille.View.overlay/2","doc":"* `padding` (optional) - Integer number of units of padding","ref":"Ratatouille.View.html#overlay/2-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.overlay/2","doc":"* canvas\n* chart\n* label\n* panel\n* row\n* sparkline\n* table\n* tree\n* viewport\n* progress_bar","ref":"Ratatouille.View.html#overlay/2-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.overlay/2","doc":"overlay([key: value], [elem1, elem2])","ref":"Ratatouille.View.html#overlay/2-examples"},{"type":"macro","title":"Ratatouille.View.panel/0","doc":"Defines an element with the `:panel` tag.","ref":"Ratatouille.View.html#panel/0"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.panel/0","doc":"* canvas\n* chart\n* label\n* panel\n* row\n* sparkline\n* table\n* tree\n* viewport\n* progress_bar","ref":"Ratatouille.View.html#panel/0-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.panel/0","doc":"Empty element:\n\n    panel()\n\nWith a block:\n\n    panel do\n      # ...child elements...\n    end","ref":"Ratatouille.View.html#panel/0-examples"},{"type":"macro","title":"Ratatouille.View.panel/1","doc":"Defines an element with the `:panel` tag and either:\n\n* given attributes and an optional block\n* a list of child elements","ref":"Ratatouille.View.html#panel/1"},{"type":"macro","title":"Attributes - Ratatouille.View.panel/1","doc":"* `color` (optional) - Color of title\n* `background` (optional) - Background of title\n* `attributes` (optional) - Attributes for the title\n* `padding` (optional) - Integer providing inner padding to use when rendering child elements\n* `height` (optional) - Height of the table in rows or `:fill` to fill the parent container's box\n* `title` (optional) - Binary containing the title for the panel\n* `border` (optional) - Attributes for the border","ref":"Ratatouille.View.html#panel/1-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.panel/1","doc":"* canvas\n* chart\n* label\n* panel\n* row\n* sparkline\n* table\n* tree\n* viewport\n* progress_bar","ref":"Ratatouille.View.html#panel/1-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.panel/1","doc":"Passing attributes:\n\n    panel(key: value)\n\nPassing attributes and a block:\n\n    panel(key: value) do\n      # ...child elements...\n    end\n\nPassing list of children:\n\n    panel([elem1, elem2])","ref":"Ratatouille.View.html#panel/1-examples"},{"type":"macro","title":"Ratatouille.View.panel/2","doc":"Defines an element with the `:panel` tag and the given attributes and\nchild elements.","ref":"Ratatouille.View.html#panel/2"},{"type":"macro","title":"Attributes - Ratatouille.View.panel/2","doc":"* `color` (optional) - Color of title\n* `background` (optional) - Background of title\n* `attributes` (optional) - Attributes for the title\n* `padding` (optional) - Integer providing inner padding to use when rendering child elements\n* `height` (optional) - Height of the table in rows or `:fill` to fill the parent container's box\n* `title` (optional) - Binary containing the title for the panel\n* `border` (optional) - Attributes for the border","ref":"Ratatouille.View.html#panel/2-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.panel/2","doc":"* canvas\n* chart\n* label\n* panel\n* row\n* sparkline\n* table\n* tree\n* viewport\n* progress_bar","ref":"Ratatouille.View.html#panel/2-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.panel/2","doc":"panel([key: value], [elem1, elem2])","ref":"Ratatouille.View.html#panel/2-examples"},{"type":"macro","title":"Ratatouille.View.progress_bar/0","doc":"Defines an element with the `:progress_bar` tag.","ref":"Ratatouille.View.html#progress_bar/0"},{"type":"macro","title":"Examples - Ratatouille.View.progress_bar/0","doc":"Empty element:\n\n    progress_bar()","ref":"Ratatouille.View.html#progress_bar/0-examples"},{"type":"macro","title":"Ratatouille.View.progress_bar/1","doc":"Defines an element with the `:progress_bar` tag and the given attributes.","ref":"Ratatouille.View.html#progress_bar/1"},{"type":"macro","title":"Attributes - Ratatouille.View.progress_bar/1","doc":"* `percentage` (required) - The actual percentage\n* `text_position` (optional) - Where to put the text showing the percentage, `:none`, `:left` or `:right`. Defaults to `:right`\n* `on_color` (optional) - The main color of the bar\n* `off_color` (optional) - The background color of the bar\n* `text_color` (optional) - The color of the percentage text","ref":"Ratatouille.View.html#progress_bar/1-attributes"},{"type":"macro","title":"Examples - Ratatouille.View.progress_bar/1","doc":"progress_bar(key: value)","ref":"Ratatouille.View.html#progress_bar/1-examples"},{"type":"macro","title":"Ratatouille.View.row/0","doc":"Defines an element with the `:row` tag.","ref":"Ratatouille.View.html#row/0"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.row/0","doc":"* column","ref":"Ratatouille.View.html#row/0-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.row/0","doc":"Empty element:\n\n    row()\n\nWith a block:\n\n    row do\n      # ...child elements...\n    end","ref":"Ratatouille.View.html#row/0-examples"},{"type":"macro","title":"Ratatouille.View.row/1","doc":"Defines an element with the `:row` tag and either:\n\n* given attributes and an optional block\n* a list of child elements","ref":"Ratatouille.View.html#row/1"},{"type":"macro","title":"Attributes - Ratatouille.View.row/1","doc":"None","ref":"Ratatouille.View.html#row/1-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.row/1","doc":"* column","ref":"Ratatouille.View.html#row/1-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.row/1","doc":"Passing attributes:\n\n    row(key: value)\n\nPassing attributes and a block:\n\n    row(key: value) do\n      # ...child elements...\n    end\n\nPassing list of children:\n\n    row([elem1, elem2])","ref":"Ratatouille.View.html#row/1-examples"},{"type":"macro","title":"Ratatouille.View.row/2","doc":"Defines an element with the `:row` tag and the given attributes and\nchild elements.","ref":"Ratatouille.View.html#row/2"},{"type":"macro","title":"Attributes - Ratatouille.View.row/2","doc":"None","ref":"Ratatouille.View.html#row/2-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.row/2","doc":"* column","ref":"Ratatouille.View.html#row/2-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.row/2","doc":"row([key: value], [elem1, elem2])","ref":"Ratatouille.View.html#row/2-examples"},{"type":"macro","title":"Ratatouille.View.sparkline/0","doc":"Defines an element with the `:sparkline` tag.","ref":"Ratatouille.View.html#sparkline/0"},{"type":"macro","title":"Examples - Ratatouille.View.sparkline/0","doc":"Empty element:\n\n    sparkline()","ref":"Ratatouille.View.html#sparkline/0-examples"},{"type":"macro","title":"Ratatouille.View.sparkline/1","doc":"Defines an element with the `:sparkline` tag and the given attributes.","ref":"Ratatouille.View.html#sparkline/1"},{"type":"macro","title":"Attributes - Ratatouille.View.sparkline/1","doc":"* `series` (required) - List of float or integer values representing the series","ref":"Ratatouille.View.html#sparkline/1-attributes"},{"type":"macro","title":"Examples - Ratatouille.View.sparkline/1","doc":"sparkline(key: value)","ref":"Ratatouille.View.html#sparkline/1-examples"},{"type":"macro","title":"Ratatouille.View.table/0","doc":"Defines an element with the `:table` tag.","ref":"Ratatouille.View.html#table/0"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.table/0","doc":"* table_row","ref":"Ratatouille.View.html#table/0-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.table/0","doc":"Empty element:\n\n    table()\n\nWith a block:\n\n    table do\n      # ...child elements...\n    end","ref":"Ratatouille.View.html#table/0-examples"},{"type":"macro","title":"Ratatouille.View.table/1","doc":"Defines an element with the `:table` tag and either:\n\n* given attributes and an optional block\n* a list of child elements","ref":"Ratatouille.View.html#table/1"},{"type":"macro","title":"Attributes - Ratatouille.View.table/1","doc":"None","ref":"Ratatouille.View.html#table/1-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.table/1","doc":"* table_row","ref":"Ratatouille.View.html#table/1-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.table/1","doc":"Passing attributes:\n\n    table(key: value)\n\nPassing attributes and a block:\n\n    table(key: value) do\n      # ...child elements...\n    end\n\nPassing list of children:\n\n    table([elem1, elem2])","ref":"Ratatouille.View.html#table/1-examples"},{"type":"macro","title":"Ratatouille.View.table/2","doc":"Defines an element with the `:table` tag and the given attributes and\nchild elements.","ref":"Ratatouille.View.html#table/2"},{"type":"macro","title":"Attributes - Ratatouille.View.table/2","doc":"None","ref":"Ratatouille.View.html#table/2-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.table/2","doc":"* table_row","ref":"Ratatouille.View.html#table/2-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.table/2","doc":"table([key: value], [elem1, elem2])","ref":"Ratatouille.View.html#table/2-examples"},{"type":"macro","title":"Ratatouille.View.table_cell/0","doc":"Defines an element with the `:table_cell` tag.","ref":"Ratatouille.View.html#table_cell/0"},{"type":"macro","title":"Examples - Ratatouille.View.table_cell/0","doc":"Empty element:\n\n    table_cell()","ref":"Ratatouille.View.html#table_cell/0-examples"},{"type":"macro","title":"Ratatouille.View.table_cell/1","doc":"Defines an element with the `:table_cell` tag and the given attributes.","ref":"Ratatouille.View.html#table_cell/1"},{"type":"macro","title":"Attributes - Ratatouille.View.table_cell/1","doc":"* `content` (required) - Binary containing the text content to be displayed\n* `color` (optional) - Constant representing color to use for foreground\n* `background` (optional) - Constant representing color to use for background\n* `attributes` (optional) - Constant representing style attributes to apply","ref":"Ratatouille.View.html#table_cell/1-attributes"},{"type":"macro","title":"Examples - Ratatouille.View.table_cell/1","doc":"table_cell(key: value)","ref":"Ratatouille.View.html#table_cell/1-examples"},{"type":"macro","title":"Ratatouille.View.table_row/0","doc":"Defines an element with the `:table_row` tag.","ref":"Ratatouille.View.html#table_row/0"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.table_row/0","doc":"* table_cell","ref":"Ratatouille.View.html#table_row/0-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.table_row/0","doc":"Empty element:\n\n    table_row()\n\nWith a block:\n\n    table_row do\n      # ...child elements...\n    end","ref":"Ratatouille.View.html#table_row/0-examples"},{"type":"macro","title":"Ratatouille.View.table_row/1","doc":"Defines an element with the `:table_row` tag and either:\n\n* given attributes and an optional block\n* a list of child elements","ref":"Ratatouille.View.html#table_row/1"},{"type":"macro","title":"Attributes - Ratatouille.View.table_row/1","doc":"* `color` (optional) - Constant representing color to use for foreground\n* `background` (optional) - Constant representing color to use for background\n* `attributes` (optional) - Constant representing style attributes to apply","ref":"Ratatouille.View.html#table_row/1-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.table_row/1","doc":"* table_cell","ref":"Ratatouille.View.html#table_row/1-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.table_row/1","doc":"Passing attributes:\n\n    table_row(key: value)\n\nPassing attributes and a block:\n\n    table_row(key: value) do\n      # ...child elements...\n    end\n\nPassing list of children:\n\n    table_row([elem1, elem2])","ref":"Ratatouille.View.html#table_row/1-examples"},{"type":"macro","title":"Ratatouille.View.table_row/2","doc":"Defines an element with the `:table_row` tag and the given attributes and\nchild elements.","ref":"Ratatouille.View.html#table_row/2"},{"type":"macro","title":"Attributes - Ratatouille.View.table_row/2","doc":"* `color` (optional) - Constant representing color to use for foreground\n* `background` (optional) - Constant representing color to use for background\n* `attributes` (optional) - Constant representing style attributes to apply","ref":"Ratatouille.View.html#table_row/2-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.table_row/2","doc":"* table_cell","ref":"Ratatouille.View.html#table_row/2-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.table_row/2","doc":"table_row([key: value], [elem1, elem2])","ref":"Ratatouille.View.html#table_row/2-examples"},{"type":"macro","title":"Ratatouille.View.text/0","doc":"Defines an element with the `:text` tag.","ref":"Ratatouille.View.html#text/0"},{"type":"macro","title":"Examples - Ratatouille.View.text/0","doc":"Empty element:\n\n    text()","ref":"Ratatouille.View.html#text/0-examples"},{"type":"macro","title":"Ratatouille.View.text/1","doc":"Defines an element with the `:text` tag and the given attributes.","ref":"Ratatouille.View.html#text/1"},{"type":"macro","title":"Attributes - Ratatouille.View.text/1","doc":"* `content` (required) - Binary containing the text content to be displayed\n* `color` (optional) - Constant representing color to use for foreground\n* `background` (optional) - Constant representing color to use for background\n* `attributes` (optional) - Constant representing style attributes to apply","ref":"Ratatouille.View.html#text/1-attributes"},{"type":"macro","title":"Examples - Ratatouille.View.text/1","doc":"text(key: value)","ref":"Ratatouille.View.html#text/1-examples"},{"type":"macro","title":"Ratatouille.View.tree/0","doc":"Defines an element with the `:tree` tag.","ref":"Ratatouille.View.html#tree/0"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.tree/0","doc":"* tree_node","ref":"Ratatouille.View.html#tree/0-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.tree/0","doc":"Empty element:\n\n    tree()\n\nWith a block:\n\n    tree do\n      # ...child elements...\n    end","ref":"Ratatouille.View.html#tree/0-examples"},{"type":"macro","title":"Ratatouille.View.tree/1","doc":"Defines an element with the `:tree` tag and either:\n\n* given attributes and an optional block\n* a list of child elements","ref":"Ratatouille.View.html#tree/1"},{"type":"macro","title":"Attributes - Ratatouille.View.tree/1","doc":"None","ref":"Ratatouille.View.html#tree/1-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.tree/1","doc":"* tree_node","ref":"Ratatouille.View.html#tree/1-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.tree/1","doc":"Passing attributes:\n\n    tree(key: value)\n\nPassing attributes and a block:\n\n    tree(key: value) do\n      # ...child elements...\n    end\n\nPassing list of children:\n\n    tree([elem1, elem2])","ref":"Ratatouille.View.html#tree/1-examples"},{"type":"macro","title":"Ratatouille.View.tree/2","doc":"Defines an element with the `:tree` tag and the given attributes and\nchild elements.","ref":"Ratatouille.View.html#tree/2"},{"type":"macro","title":"Attributes - Ratatouille.View.tree/2","doc":"None","ref":"Ratatouille.View.html#tree/2-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.tree/2","doc":"* tree_node","ref":"Ratatouille.View.html#tree/2-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.tree/2","doc":"tree([key: value], [elem1, elem2])","ref":"Ratatouille.View.html#tree/2-examples"},{"type":"macro","title":"Ratatouille.View.tree_node/0","doc":"Defines an element with the `:tree_node` tag.","ref":"Ratatouille.View.html#tree_node/0"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.tree_node/0","doc":"* tree_node","ref":"Ratatouille.View.html#tree_node/0-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.tree_node/0","doc":"Empty element:\n\n    tree_node()\n\nWith a block:\n\n    tree_node do\n      # ...child elements...\n    end","ref":"Ratatouille.View.html#tree_node/0-examples"},{"type":"macro","title":"Ratatouille.View.tree_node/1","doc":"Defines an element with the `:tree_node` tag and either:\n\n* given attributes and an optional block\n* a list of child elements","ref":"Ratatouille.View.html#tree_node/1"},{"type":"macro","title":"Attributes - Ratatouille.View.tree_node/1","doc":"* `content` (required) - Binary label for the node\n* `color` (optional) - Constant representing color to use for foreground\n* `background` (optional) - Constant representing color to use for background\n* `attributes` (optional) - Constant representing style attributes to apply","ref":"Ratatouille.View.html#tree_node/1-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.tree_node/1","doc":"* tree_node","ref":"Ratatouille.View.html#tree_node/1-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.tree_node/1","doc":"Passing attributes:\n\n    tree_node(key: value)\n\nPassing attributes and a block:\n\n    tree_node(key: value) do\n      # ...child elements...\n    end\n\nPassing list of children:\n\n    tree_node([elem1, elem2])","ref":"Ratatouille.View.html#tree_node/1-examples"},{"type":"macro","title":"Ratatouille.View.tree_node/2","doc":"Defines an element with the `:tree_node` tag and the given attributes and\nchild elements.","ref":"Ratatouille.View.html#tree_node/2"},{"type":"macro","title":"Attributes - Ratatouille.View.tree_node/2","doc":"* `content` (required) - Binary label for the node\n* `color` (optional) - Constant representing color to use for foreground\n* `background` (optional) - Constant representing color to use for background\n* `attributes` (optional) - Constant representing style attributes to apply","ref":"Ratatouille.View.html#tree_node/2-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.tree_node/2","doc":"* tree_node","ref":"Ratatouille.View.html#tree_node/2-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.tree_node/2","doc":"tree_node([key: value], [elem1, elem2])","ref":"Ratatouille.View.html#tree_node/2-examples"},{"type":"macro","title":"Ratatouille.View.view/0","doc":"Defines an element with the `:view` tag.","ref":"Ratatouille.View.html#view/0"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.view/0","doc":"* overlay\n* canvas\n* chart\n* label\n* panel\n* row\n* sparkline\n* table\n* tree\n* viewport\n* progress_bar","ref":"Ratatouille.View.html#view/0-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.view/0","doc":"Empty element:\n\n    view()\n\nWith a block:\n\n    view do\n      # ...child elements...\n    end","ref":"Ratatouille.View.html#view/0-examples"},{"type":"macro","title":"Ratatouille.View.view/1","doc":"Defines an element with the `:view` tag and either:\n\n* given attributes and an optional block\n* a list of child elements","ref":"Ratatouille.View.html#view/1"},{"type":"macro","title":"Attributes - Ratatouille.View.view/1","doc":"* `top_bar` (optional) - A `:bar` element to occupy the view's first row\n* `bottom_bar` (optional) - A `:bar` element to occupy the view's last row","ref":"Ratatouille.View.html#view/1-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.view/1","doc":"* overlay\n* canvas\n* chart\n* label\n* panel\n* row\n* sparkline\n* table\n* tree\n* viewport\n* progress_bar","ref":"Ratatouille.View.html#view/1-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.view/1","doc":"Passing attributes:\n\n    view(key: value)\n\nPassing attributes and a block:\n\n    view(key: value) do\n      # ...child elements...\n    end\n\nPassing list of children:\n\n    view([elem1, elem2])","ref":"Ratatouille.View.html#view/1-examples"},{"type":"macro","title":"Ratatouille.View.view/2","doc":"Defines an element with the `:view` tag and the given attributes and\nchild elements.","ref":"Ratatouille.View.html#view/2"},{"type":"macro","title":"Attributes - Ratatouille.View.view/2","doc":"* `top_bar` (optional) - A `:bar` element to occupy the view's first row\n* `bottom_bar` (optional) - A `:bar` element to occupy the view's last row","ref":"Ratatouille.View.html#view/2-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.view/2","doc":"* overlay\n* canvas\n* chart\n* label\n* panel\n* row\n* sparkline\n* table\n* tree\n* viewport\n* progress_bar","ref":"Ratatouille.View.html#view/2-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.view/2","doc":"view([key: value], [elem1, elem2])","ref":"Ratatouille.View.html#view/2-examples"},{"type":"macro","title":"Ratatouille.View.viewport/0","doc":"Defines an element with the `:viewport` tag.","ref":"Ratatouille.View.html#viewport/0"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.viewport/0","doc":"* canvas\n* chart\n* label\n* panel\n* row\n* sparkline\n* table\n* tree\n* viewport\n* progress_bar","ref":"Ratatouille.View.html#viewport/0-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.viewport/0","doc":"Empty element:\n\n    viewport()\n\nWith a block:\n\n    viewport do\n      # ...child elements...\n    end","ref":"Ratatouille.View.html#viewport/0-examples"},{"type":"macro","title":"Ratatouille.View.viewport/1","doc":"Defines an element with the `:viewport` tag and either:\n\n* given attributes and an optional block\n* a list of child elements","ref":"Ratatouille.View.html#viewport/1"},{"type":"macro","title":"Attributes - Ratatouille.View.viewport/1","doc":"* `offset_x` (optional) - Integer representing the number of columns to offset the child content by. Defaults to 0.\n* `offset_y` (optional) - Integer representing the number of rows to offset the child content by. Defaults to 0.","ref":"Ratatouille.View.html#viewport/1-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.viewport/1","doc":"* canvas\n* chart\n* label\n* panel\n* row\n* sparkline\n* table\n* tree\n* viewport\n* progress_bar","ref":"Ratatouille.View.html#viewport/1-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.viewport/1","doc":"Passing attributes:\n\n    viewport(key: value)\n\nPassing attributes and a block:\n\n    viewport(key: value) do\n      # ...child elements...\n    end\n\nPassing list of children:\n\n    viewport([elem1, elem2])","ref":"Ratatouille.View.html#viewport/1-examples"},{"type":"macro","title":"Ratatouille.View.viewport/2","doc":"Defines an element with the `:viewport` tag and the given attributes and\nchild elements.","ref":"Ratatouille.View.html#viewport/2"},{"type":"macro","title":"Attributes - Ratatouille.View.viewport/2","doc":"* `offset_x` (optional) - Integer representing the number of columns to offset the child content by. Defaults to 0.\n* `offset_y` (optional) - Integer representing the number of rows to offset the child content by. Defaults to 0.","ref":"Ratatouille.View.html#viewport/2-attributes"},{"type":"macro","title":"Allowed Child Elements - Ratatouille.View.viewport/2","doc":"* canvas\n* chart\n* label\n* panel\n* row\n* sparkline\n* table\n* tree\n* viewport\n* progress_bar","ref":"Ratatouille.View.html#viewport/2-allowed-child-elements"},{"type":"macro","title":"Examples - Ratatouille.View.viewport/2","doc":"viewport([key: value], [elem1, elem2])","ref":"Ratatouille.View.html#viewport/2-examples"},{"type":"module","title":"Ratatouille.Window","doc":"A GenServer to manage the terminal window, along with a client API to perform\nupdates and retrieve window information.","ref":"Ratatouille.Window.html"},{"type":"function","title":"Ratatouille.Window.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Ratatouille.Window.html#child_spec/1"},{"type":"function","title":"Ratatouille.Window.close/1","doc":"Closes the window by stopping the GenServer. Prior to this, termbox is\nde-initialized so that the terminal is restored to its previous state.","ref":"Ratatouille.Window.html#close/1"},{"type":"function","title":"Ratatouille.Window.fetch/2","doc":"Fetches an attribute for the window. This is currently limited to the window\ndimensions, which can be useful when laying out content.","ref":"Ratatouille.Window.html#fetch/2"},{"type":"function","title":"Examples - Ratatouille.Window.fetch/2","doc":"iex> Window.fetch(:height)\n    {:ok, 124}\n    iex> Window.fetch(:width)\n    {:ok, 50}\n    iex> Window.fetch(:foo)\n    {:error, :unknown_attribute}","ref":"Ratatouille.Window.html#fetch/2-examples"},{"type":"type","title":"Ratatouille.Window.fetch_attr/0","doc":"","ref":"Ratatouille.Window.html#t:fetch_attr/0"},{"type":"function","title":"Ratatouille.Window.start_link/1","doc":"Starts the gen_server representing the window.\n\nThe window is intended to be run as a singleton gen_server process, as\ninitializing the underlying termbox library multiple times on the same TTY can\nlead to undefined behavior.","ref":"Ratatouille.Window.html#start_link/1"},{"type":"function","title":"Options - Ratatouille.Window.start_link/1","doc":"* `:name`        - Override the name passed to `GenServer.start_link/3`. By\n                   default, `name: Ratatouille.Window` is passed in order to\n                   protect against accidental double initialization, but this\n                   can be overridden by passing `nil` or an alternate value.\n* `:input_mode`  - Configure the input mode. See `Ratatouille.Constants.input_mode/1`\n                   for possible values.\n* `:output_mode` - Configure the output mode. See `Ratatouille.Constants.output_mode/1`\n                   for possible values.","ref":"Ratatouille.Window.html#start_link/1-options"},{"type":"function","title":"Ratatouille.Window.update/2","doc":"Updates the window by rendering the given view to the termbox buffer and\npresenting it.","ref":"Ratatouille.Window.html#update/2"},{"type":"extras","title":"Changelog","doc":"# Changelog\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]","ref":"changelog.html"},{"type":"extras","title":"Changed - Changelog","doc":"## [0.5.1] - 2020-03-24","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"* Updated build dependencies.\n\n\n## [0.5.0] - 2019-04-28","ref":"changelog.html#changed-1"},{"type":"extras","title":"Fixed - Changelog","doc":"* Possible rendering error with multi-byte UTF-8 characters.","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"* New `viewport` element for offsetting the render origin of child\n  content (e.g., to implement scrolling)\n* New `canvas` and `canvas_cell` elements for drawing arbitrary shapes\n  (see snake example).\n* Support for rendering multi-line content (with automatic line\n  wrapping) given to the `label` element.\n* Support for styling tree node content.\n* Support for styling panel title content (thanks to @iboard) and\n  configuring panel's padding.\n* Support for passing colors and text attributes directly as\n  atoms, so the integer constants no longer need to be looked up.\n* Improved documentation of element hierarchy restrictions (thanks to\n  @trescenzi)\n\n\n## [0.4.2] - 2019-03-03","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"* Possible crash in the termbox NIFs when polling for events (updates ex_termbox\n  to 1.0.0).\n\n\n## [0.4.1] - 2019-02-23","ref":"changelog.html#fixed-1"},{"type":"extras","title":"Fixed - Changelog","doc":"* Errors in panel box calculation with certain layouts.","ref":"changelog.html#fixed-2"},{"type":"extras","title":"Added - Changelog","doc":"* Documentation of element attributes is now generated based on element specs.\n\n\n## [0.4.0] - 2019-02-22","ref":"changelog.html#added-1"},{"type":"extras","title":"Added - Changelog","doc":"* Applications based on the Elm Architecture.\n  * App behaviour\n  * Runtime\n  * Runtime Supervisor\n  * Components & Subscriptions\n  * New examples","ref":"changelog.html#added-2"},{"type":"extras","title":"Changed - Changelog","doc":"* Views support labels as direct children.","ref":"changelog.html#changed-2"},{"type":"extras","title":"Removed - Changelog","doc":"* Removed the experimental component support in favor of the new TEA-based apps.\n\n\n## [0.3.0] - 2019-01-25","ref":"changelog.html#removed"},{"type":"extras","title":"Changed - Changelog","doc":"* Element attributes are now validated based on the element's spec. Some\n  attributes are optional, while others are required. It's not allowed to pass\n  attributes that are not defined in the spec.\n* The View DSL was extracted to `Ratatouille.View`. Imports like\n  `import Ratatouille.Renderer.View` should be updated to `import Ratatouille.View`.","ref":"changelog.html#changed-3"},{"type":"extras","title":"License","doc":"# MIT License\n\nCopyright (c) 2018 Nick Reynolds\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"},{"type":"extras","title":"Overview","doc":"# Ratatouille\n\n[![Build Status](https://travis-ci.org/ndreynolds/ratatouille.svg?branch=master)](https://travis-ci.org/ndreynolds/ratatouille)\n[![Module Version](https://img.shields.io/hexpm/v/ratatouille.svg)](https://hex.pm/packages/ratatouille)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/ratatouille/)\n[![Total Download](https://img.shields.io/hexpm/dt/ratatouille.svg)](https://hex.pm/packages/ratatouille)\n[![License](https://img.shields.io/hexpm/l/ratatouille.svg)](https://github.com/ndreynolds/ratatouille/blob/master/LICENSE.md)\n[![Last Updated](https://img.shields.io/github/last-commit/ndreynolds/ratatouille.svg)](https://github.com/ndreynolds/ratatouille/commits/master)\n\nRatatouille is a declarative terminal UI kit for Elixir for building rich\ntext-based terminal applications similar to how you write HTML.\n\nIt builds on top of the [termbox][termbox] API (using the Elixir bindings from\n[ex_termbox][ex_termbox]).\n\nFor the API Reference, see: [https://hexdocs.pm/ratatouille](https://hexdocs.pm/ratatouille).\n\n![Toby](./assets/toby.png)\n*[Toby](https://github.com/ndreynolds/toby), a terminal-based Erlang observer built with Ratatouille*\n\n[termbox]: https://github.com/nsf/termbox\n[ex_termbox]: https://github.com/ndreynolds/ex_termbox\n\n**Table of Contents**\n\n* [Ratatouille](#ratatouille)\n  * [Getting Started](#getting-started)\n    * [Building an Application](#building-an-application)\n      * [init/1](#init1)\n      * [update/2](#update2)\n      * [render/1](#render1)\n      * [Running it](#running-it)\n  * [Views](#views)\n    * [The DSL](#the-dsl)\n    * [Adding Logic](#adding-logic)\n    * [Styling](#styling)\n    * [Views are Strict](#views-are-strict)\n  * [Example Applications](#example-applications)\n  * [Under the Hood](#under-the-hood)\n  * [Packaging and Distributing](#packaging-and-distributing)\n    * [Defining an OTP Application](#defining-an-otp-application)\n    * [Executable Releases with Distillery](#executable-releases-with-distillery)\n  * [Projects using Ratatouille](#projects-using-ratatouille)\n  * [Installation](#installation)\n    * [From Hex](#from-hex)\n    * [From Source](#from-source)\n  * [Roadmap](#roadmap)\n  * [Contributing](#contributing)\n    * [Running the Tests](#running-the-tests)","ref":"readme.html"},{"type":"extras","title":"Getting Started - Overview","doc":"Ratatouille implements [The Elm Architecture][tea] (TEA) as a way to structure\napplication logic. This fits quite naturally in Elixir and is part of what makes\nRatatouille declarative. If you've already used TEA on the web, this should feel\nvery familiar.\n\nAs with a GenServer definition, Ratatouille apps only implement a behaviour by\ndefining callbacks and don't know how to start or run themselves. It's the\napplication runtime that handles all of those (sometimes tricky) details.\n\n[tea]: https://guide.elm-lang.org/architecture/","ref":"readme.html#getting-started"},{"type":"extras","title":"Building an Application - Overview","doc":"Let's build a simple application that displays an integer counter which can be\nincremented when the user presses `+` and decremented when the user presses `-`.\n\nFirst a quick clarification, since we're using the word \"application\" a lot. For\nour purposes, an application is a terminal application, and not necessarily an\nOTP application, but your terminal application could also be an OTP\napplication. We'll cover that in [Packaging and Distributing\nApplications](#packaging-and-distributing) below.\n\nBack to the counter app. First we'll look at the entire example, then we'll go\nthrough it line by line to see what each line does. You can also find this\nexample [in the repo][counter_example] and run it with `mix run`.\n\n[counter_example]: https://github.com/ndreynolds/ratatouille/blob/master/examples/counter.exs\n\n```elixir\n# examples/counter.exs\n\ndefmodule Counter do\n  @behaviour Ratatouille.App\n\n  import Ratatouille.View\n\n  def init(_context), do: 0\n\n  def update(model, msg) do\n    case msg do\n      {:event, %{ch: ?+}} -> model + 1\n      {:event, %{ch: ?-}} -> model - 1\n      _ -> model\n    end\n  end\n\n  def render(model) do\n    view do\n      label(content: \"Counter is #{model} (+/-)\")\n    end\n  end\nend\n\nRatatouille.run(Counter)\n```\n\nAt the top, we define a new module (`Counter`) for the app and we inform Elixir\nthat it will implement the `Ratatouille.App` behaviour. This just ensures we're\nwarned if we forget to implement a callback and serves as documentation that\nthis is a Ratatouille app.\n\n```elixir\ndefmodule Counter do\n  @behaviour Ratatouille.App\n\n  # ...\nend\n```\n\nNext, we import the View DSL from\n[`Ratatouille.View`](https://hexdocs.pm/ratatouille/Ratatouille.View):\n\n```elixir\nimport Ratatouille.View\n```\n\nThe View DSL provides element builder functions like `view`, `row`, `table`,\n`label` that you can use to define views. Think of them like HTML tags.\n\n#### `init/1`\n\nThe `init/1` callback defines the initial model. \"Model\" is the Elm\narchitecture's term for what we often call \"state\" in Elixir/Erlang. As with a\nGenServer, the state (our model) will later be passed to callbacks when things\nhappen in order to allow the app to update it.\n\nThe model can be any Erlang term. For larger apps, it's helpful to use maps or\nstructs to organize different pieces of the state. Here, we just have an integer\ncounter, so we return `0` as our initial model:\n\n```elixir\ndefmodule Counter do\n  # ...\n\n  def init(_context), do: 0\n\n  # ...\nend\n```\n\n#### `update/2`\n\nThe `update/2` callback defines how to transform the model when a particular\nmessage is received. Ratatouille's runtime will automatically call `update/2`\nwhen terminal events occur (pressing a key, resizing the window, clicking the\nmouse, etc.). We can also send ourselves messages via subscriptions and commands.\n\nHere, we'd like to increment the counter when we get a `?+` key press and\ndecrement it when get a `?-`. Event messages are based on the underlying termbox\nevents and characters are given as code points (e.g., `?a` is `97`).\n\n```elixir\ndefmodule Counter do\n  # ...\n\n  def update(model, msg) do\n    case msg do\n      {:event, %{ch: ?+}} -> model + 1\n      {:event, %{ch: ?-}} -> model - 1\n      _ -> model\n    end\n  end\n\n  # ...\nend\n```\n\nIt's a good idea to provide a fallback clause in case we don't know how to\nhandle a message. This way the app won't crash if the user presses a key that\nthe app doesn't handle. But if things stop working as you expect, try removing\nthe fallback to see if important messages are going unmatched.\n\n#### `render/1`\n\nThe `render/1` callback defines a view to display the model. The runtime will\ncall it as needed when it needs to update the terminal window.\n\nLike an HTML document, a view is defined as a tree of elements (nodes).\nElements have attributes (e.g., `text: bold`) and children (nested content).\nWhile helper functions can return arbitrary element trees, the `render/1`\ncallback must return a view tree starting with a root `view` element---it's\nsort of like the ` ` tag in HTML.\n\n```elixir\ndefmodule Counter do\n  # ...\n\n  def render(model) do\n    view do\n      label(content: \"Counter is #{model} (+/-)\")\n    end\n  end\n\n  # ...\nend\n```\n\n#### Running it\n\nThere's a final and very important line at the bottom:\n\n```elixir\nRatatouille.run(Counter)\n```\n\nThis starts the application runtime with our app definition. Options can be\npassed as a second argument. This is an easy way to run simple apps. For more\ncomplicated ones, it's recommended to define an OTP application.\n\nThat's it---now you can run the program with `mix run  `. To run the\nbundled example:\n\n```bash\n$ mix run examples/counter.exs\n```\n\nYou should see the counter we defined, be able to make changes to it with `+`\nand `-`, and be able to quit using `q`.","ref":"readme.html#building-an-application"},{"type":"extras","title":"Views - Overview","doc":"Ratatouille's views are trees of elements similar to HTML in structure. For\nexample, here's how to define a two-column layout:\n\n```elixir\nview do\n  row do\n    column size: 6 do\n      panel title: \"Left Column\" do\n        label(content: \"Text on the left\")\n      end\n    end\n\n    column size: 6 do\n      panel title: \"Right Column\" do\n        label(content: \"Text on the right\")\n      end\n    end\n  end\nend\n```","ref":"readme.html#views"},{"type":"extras","title":"The DSL - Overview","doc":"As you might have noticed, Ratatouille provides a small DSL on top of Elixir for\ndefining views. These are functions and macros which accept attributes and/or\nchild elements in different formats. For example, a `column` element can be\ndefined in all of the following ways:\n\n```elixir\ncolumn()\n\ncolumn(size: 12)\n\ncolumn do\n  # ... child elements ...\nend\n\ncolumn size: 12 do\n  # ... child elements ...\nend\n```\n\nAll of these evaluate to a `%Ratatouille.Renderer.Element{tag: :column}` struct.\nThe macros provide syntactic sugar, but under the hood it's all structs.\n\nHere's a list of all the elements provided by `Ratatouille.View`:\n\n| Element | Description |\n| ------- | ----------- |\n| [`bar`](https://hexdocs.pm/ratatouille/Ratatouille.View.html#bar/0) | Block-level element for creating title, status or menu bars |\n| [`canvas`](https://hexdocs.pm/ratatouille/Ratatouille.View.html#canvas/0) | A free-form canvas for drawing arbitrary shapes |\n| [`canvas_cell`](https://hexdocs.pm/ratatouille/Ratatouille.View.html#canvas_cell/0) | A canvas cell which represents one square of the canvas |\n| [`chart`](https://hexdocs.pm/ratatouille/Ratatouille.View.html#chart/0) | Element for plotting a series as a multi-line chart |\n| [`column`](https://hexdocs.pm/ratatouille/Ratatouille.View.html#column/0) | Container occupying a vertical segment of the grid |\n| [`label`](https://hexdocs.pm/ratatouille/Ratatouille.View.html#label/0) | Block-level element for displaying text |\n| [`overlay`](https://hexdocs.pm/ratatouille/Ratatouille.View.html#overlay/0) | Container overlaid on top of the view |\n| [`panel`](https://hexdocs.pm/ratatouille/Ratatouille.View.html#panel/0) | Container with a border and title used to demarcate content |\n| [`row`](https://hexdocs.pm/ratatouille/Ratatouille.View.html#row/0) | Container used to define grid layouts with one or more columns |\n| [`sparkline`](https://hexdocs.pm/ratatouille/Ratatouille.View.html#sparkline/0) | Element for plotting a series in a single line |\n| [`table`](https://hexdocs.pm/ratatouille/Ratatouille.View.html#table/0) | Container for displaying data in rows and columns |\n| [`table_cell`](https://hexdocs.pm/ratatouille/Ratatouille.View.html#table_cell/0) | Element representing a table cell |\n| [`table_row`](https://hexdocs.pm/ratatouille/Ratatouille.View.html#table_row/0) | Container representing a row of the table |\n| [`text`](https://hexdocs.pm/ratatouille/Ratatouille.View.html#text/0) | Inline element for displaying uniformly-styled text |\n| [`tree`](https://hexdocs.pm/ratatouille/Ratatouille.View.html#tree/0) | Container for displaying data as a tree of nodes |\n| [`tree_node`](https://hexdocs.pm/ratatouille/Ratatouille.View.html#tree_node/0) | Container representing a tree node |\n| [`view`](https://hexdocs.pm/ratatouille/Ratatouille.View.html#view/0) | Top-level container |","ref":"readme.html#the-dsl"},{"type":"extras","title":"Adding Logic - Overview","doc":"Because it's just Elixir code, you can freely mix in Elixir syntax and abstract\nviews using functions:\n\n```elixir\nlabel(content: a_variable)\n\nview do\n  case current_tab do\n    :one -> render_tab_one()\n    :two -> render_tab_two()\n  end\nend\n```\n\n```elixir\nif window.width > 80 do\n  row do\n    column(size: 6)\n    column(size: 6)\n  end\nelse\n  row do\n    column(size: 12)\n  end\nend\n```","ref":"readme.html#adding-logic"},{"type":"extras","title":"Styling - Overview","doc":"Attributes are used to style text and other content:\n\n```elixir\n# Labels are block-level, so this makes text within the whole block red.\nlabel(content: \"Red text\", color: :red)\n\n# Nested inline text elements can be used to style differently within a label.\nlabel do\n  text(content: \"R\", color: :red)\n  text(content: \"G\", color: :green)\n  text(content: \"B\", color: :blue)\nend\n\n# `color` sets the foreground, while `background` sets the background.\nlabel(content: \"Black on white\", color: :black, background: :white)\n\n# `attributes` accepts a list of text attributes, here `:bold` and `:underline`.\nlabel(content: \"Bold and underlined text\", attributes: [:bold, :underline])\n```\n\nStyling is still being developed, so it's not currently possible to style every\naspect of every element, but this will improve with time.","ref":"readme.html#styling"},{"type":"extras","title":"Views are Strict - Overview","doc":"Most web browsers will happily try to make sense of any HTML you give them. For\nexample, you can put a `td` directly under a `div` and the content will likely\nstill be rendered.\n\nRatatouille takes a different, more strict approach and first validates that the\nview tree is well-structured. If it's not valid, an error is raised explaining\nthe problem. This is intended to provide quick feedback when something's wrong.\nRestricting the set of valid views also helps to simplify the rendering\nimplementation.\n\nIt's helpful to keep the following things in mind when defining views:\n\n* Each tag has a list of allowed child tags. For example, a `row` may only have\n  elements with the `column` tag as direct descendants.\n* Each tag has a list of attributes. Some attributes are required, and these\n  must be set. Optional attributes have some default behavior when unset. It's\n  not allowed to set an attribute that's not in the list.\n* A `view` element must be the root element of any view tree you'd like to\n  render.\n\nSee the list of elements above for documentation on each element.","ref":"readme.html#views-are-strict"},{"type":"extras","title":"Example Applications - Overview","doc":"The following examples show off different aspects of the framework:\n\n| Name | Description |\n| ---- | ----------- |\n| [`rendering.exs`](https://github.com/ndreynolds/ratatouille/tree/master/examples/rendering.exs) | A rendering demo of all the supported elements |\n| [`counter.exs`](https://github.com/ndreynolds/ratatouille/tree/master/examples/counter.exs) | How to create a simple app with state and updates |\n| [`editor.exs`](https://github.com/ndreynolds/ratatouille/tree/master/examples/editor.exs) | How to use receive and display user input |\n| [`multiple_views.exs`](https://github.com/ndreynolds/ratatouille/tree/master/examples/multiple_views.exs) | How to render different views/tabs based on a selection |\n| [`subscriptions.exs`](https://github.com/ndreynolds/ratatouille/tree/master/examples/subscriptions.exs) | How to subscribe to multiple intervals |\n| [`commands.exs`](https://github.com/ndreynolds/ratatouille/tree/master/examples/commands.exs) | How to run commands asynchronously and receive the results |\n| [`snake.exs`](https://github.com/ndreynolds/ratatouille/tree/master/examples/snake.exs) | How to make a simple game |\n| [`documentation_browser.exs`](https://github.com/ndreynolds/ratatouille/tree/master/examples/documentation_browser.exs) | How to render and scroll multiline content |\n\nWith the repository cloned locally, run an example with\n`mix run examples/ .exs`. Examples can be quit with `q` or `CTRL-c`\n(unless indicated otherwise).","ref":"readme.html#example-applications"},{"type":"extras","title":"Under the Hood - Overview","doc":"The application runtime abstracts away many of the details concerning how the\nterminal window is updated and how events are received. If you're interested in\nhow these things actually work, or if the runtime doesn't support your use case,\nsee this guide:","ref":"readme.html#under-the-hood"},{"type":"extras","title":"Packaging and Distributing - Overview","doc":"*Warning: This part is still rough around the edges.*\n\nWhile it's easy to run apps while developing with `mix run`, packaging them for\nothers to easily run is a bit more complicated. Depending on the type of app\nyou're building, it might not be reasonable to assume that users have any\nElixir or Erlang tools installed. Terminal apps are usually distributed as\nbinary executables so that they can just be run as such without additional\ndependencies. Fortunately, this is possible using OTP releases that bundle\nERTS.","ref":"readme.html#packaging-and-distributing"},{"type":"extras","title":"Defining an OTP Application - Overview","doc":"In order to create an OTP release, we first need to define an OTP application\nthat runs the terminal application. `Ratatouille.Runtime.Supervisor` takes care\nof starting all the necessary runtime components, so we start this supervisor\nunder the OTP application supervisor and pass it a Ratatouille app definition\n(along with any other runtime configuration).\n\nFor example, the OTP application for toby looks like this:\n\n```elixir\ndefmodule Toby do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      {Ratatouille.Runtime.Supervisor, runtime: [app: Toby.App]},\n      # other workers...\n    ]\n\n    Supervisor.start_link(\n      children,\n      strategy: :one_for_one,\n      name: Toby.Supervisor\n    )\n  end\nend\n```","ref":"readme.html#defining-an-otp-application"},{"type":"extras","title":"Executable Releases with Distillery - Overview","doc":"We'll use Distillery to create the OTP release, as it can even create\ndistributable, self-contained executables. Releases built on a given\narchitecture can generally be run on machines of the same architecture.\n\nFollow the Distillery guide to generate a release configuration:\n\n \n\nIn order to make a \"batteries-included\" release, it's important that you have\n`include_erts` set to `true`:\n\n``` elixir\nenvironment :prod do\n  # ...\n  set(include_erts: true)\n  # ...\nend\n```\n\nNow it's possible to generate the release:\n\n```bash\nMIX_ENV=prod mix release --executable --transient\n```\n\nThis creates a Distillery release that bundles the Erlang runtime and the\napplication. Start it in the foreground, e.g.:\n\n```bash\n_build/prod/rel/toby/bin/toby.run foreground\n```\n\nYou can also move this executable somewhere else (e.g., to a directory in your\n$PATH). A current caveat is that it must be able to unpack itself, as Distillery\nexecutables are self-extracting archives.","ref":"readme.html#executable-releases-with-distillery"},{"type":"extras","title":"Projects using Ratatouille - Overview","doc":"For inspiration or ideas on how to structure your application, check out this list of\nprojects built with Ratatouille:\n\n* [`tefter/cli`](https://github.com/tefter/cli) - the command-line client for Tefter\n* [`toby`](https://github.com/ndreynolds/toby) - a terminal-based Erlang observer\n\nIf you have a project you'd like to include here, just open a PR to add it to the list.","ref":"readme.html#projects-using-ratatouille"},{"type":"extras","title":"Installation - Overview","doc":"","ref":"readme.html#installation"},{"type":"extras","title":"From Hex - Overview","doc":"Add Ratatouille as a dependency in your project's `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ratatouille, \"~> 0.5.0\"}\n  ]\nend\n```","ref":"readme.html#from-hex"},{"type":"extras","title":"From Source - Overview","doc":"To try out the master branch, first clone the repo:\n\n```bash\ngit clone https://github.com/ndreynolds/ratatouille.git\ncd ratatouille\n```\n\nNext, fetch the deps:\n\n```\nmix deps.get\n```\n\nFinally, try out one of the included [`examples/`][examples]:\n\n```\nmix run examples/rendering.exs\n```\n\nIf you see lots of things drawn on your terminal screen, you're good to go. Use\n\"q\" to quit in the examples (unless otherwise specified).\n\n[examples]: https://github.com/ndreynolds/ratatouille/blob/master/examples/","ref":"readme.html#from-source"},{"type":"extras","title":"Roadmap - Overview","doc":"* Apps\n  * [x] Application Runtime\n  * [x] Subscriptions\n  * [x] Commands\n* Views / Rendering\n  * [x] Rendering engine with basic elements\n  * [ ] More configurable charts (axis label, color, multiple lines, etc.)\n  * [ ] Uniform support for text styling (incl. inheritance)\n  * [x] Automatic translation to termbox styling constants\n    * For example, `color: :red` instead of `color: Constants.color(:red)`.\n  * [ ] Rendering optimizations (view diffing, more efficient updates, etc.)\n* Events\n  * [ ] Translate termbox events to a cleaner format\n    * Dealing with the integer constants is inconvenient. These could be\n      converted to atoms by the event manager.\n* Terminal Backend\n  * [x] ex_termbox NIFs\n  * [ ] Alternative port-based termbox backend\n* Customization\n  * [ ] Registering custom element renderers\n    * This would support using custom elements (e.g. `my_table()`) that are\n      defined outside of the core library.","ref":"readme.html#roadmap"},{"type":"extras","title":"Contributing - Overview","doc":"Contributions are much appreciated. They don't necessarily have to come in the\nform of code, I'm also very thankful for bug reports, documentation\nimprovements, questions, and suggestions.","ref":"readme.html#contributing"},{"type":"extras","title":"Running the Tests - Overview","doc":"Run the unit tests as usual:\n\n```\nmix test\n```\n\nRatatouille also includes integration tests of the bundled examples. These\naren't included in the default suite because they actually run the example apps.\nThe integration suite can be run like so:\n\n```\nmix test --only integration\n```","ref":"readme.html#running-the-tests"},{"type":"extras","title":"Copyright and License - Overview","doc":"Copyright (c) 2018 Nick Reynolds\n\nThis software is released under the [MIT License](./LICENSE.md).","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"Under the Hood","doc":"# Under the Hood\n\nRatatouille's runtime wraps up most of the logic concerning the application\nloop, update the terminal window, and subscribing to and delegating events.\nIt's convenient to let the runtime worry about these details, and it allows us\nto write apps more declaratively by defining them in terms of\ncallbacks---similar to how we implement OTP behaviours like gen_server and\nsupervisor.\n\nIn some cases, it may be necessary to control these details. This guide\nexplains how you can use the window, event manager, `receive` and some\nrecursion to manually define an application loop.","ref":"under-the-hood.html"},{"type":"extras","title":"Hello World from Scratch - Under the Hood","doc":"Let's build a hello world application. It'll display \"Hello World\" and quit when\nthe \"q\" key is pressed. First we'll look at the entire example, then we'll go\nthrough it line by line to see what each line does. You can also find this\nexample [in the repo][hello_world_example] and run it with `mix run`.\n\n[hello_world_example]: https://github.com/ndreynolds/ratatouille/blob/master/examples/without-runtime/hello_world.exs\n\n```elixir\n# examples/hello_world.exs\n\nalias Ratatouille.{EventManager, Window}\n\nimport Ratatouille.View\n\n{:ok, _pid} = Window.start_link()\n{:ok, _pid} = EventManager.start_link()\n:ok = EventManager.subscribe(self())\n\nhello_world_view =\n  view do\n    panel title: \"Hello, World!\", height: :fill do\n      label(content: \"Press 'q' to quit.\")\n    end\n  end\n\n:ok = Window.update(hello_world_view)\n\nreceive do\n  {:event, %{ch: ?q}} ->\n    :ok = Window.close()\nend\n```\n\nFirst, some aliases for the modules we'll use:\n\n```elixir\nalias Ratatouille.{EventManager, Window}\n```\n\nNext, we import the View DSL from\n[`Ratatouille.View`](https://hexdocs.pm/ratatouille/Ratatouille.View):\n\n```elixir\nimport Ratatouille.View\n```\n\nThe View DSL provides element builder functions like `view`, `row`, `table`,\n`label` that you can use to define views. Think of them like HTML tags.\n\nNow we'll initialize the application using `Ratatouille.Window`. This is a\ngen_server that manages our terminal window and exposes a basic API for\naccessing information about or updating the terminal window. On init, it draws a\nblank canvas over the terminal:\n\n```elixir\n{:ok, _pid} = Window.start_link()\n```\n\nIn a real project, you'll usually want to use an OTP application with a proper\nsupervision tree, but here we'll keep it as simple as possible and start our\nprocesses manually.\n\nIn order to react to keyboard, click or resize events, we'll use\n`Ratatouille.EventManager`. The event manager allows processes to subscribe to\nevents and then send its subscribers a message whenever an event is triggered.\nWe need to start the event manager and subscribe the current process to any\nevents:\n\n```elixir\n{:ok, _pid} = EventManager.start_link()\n:ok = EventManager.subscribe(self())\n```\n\nNext, we define a view. Similar to HTML, a view is defined as a tree of nodes.\nNodes have attributes (e.g., text: bold) and children (nested content). Every\nview must start with a root `view` element---it's sort of like the ` ` tag\nin HTML.\n\n```elixir\nhello_world_view =\n  view do\n    panel title: \"Hello, World!\", height: :fill do\n      label(content: \"Press 'q' to quit.\")\n    end\n  end\n```\n\nDefining a view only does just that. To render it to the screen, we need to call\nthe `Window.update/1` function, passing our view as the argument.\n\n```elixir\n:ok = Window.update(hello_world_view)\n```\n\nWhen a key is pressed, it'll be sent to us by the event manager. Once we receive\na 'q' key press, we'll close the application. Here, we use the built-in\n`receive` function with pattern-matching in order to match only the 'q' key\npress event:\n\n```elixir\nreceive do\n  {:event, %{ch: ?q}} ->\n    :ok = Window.close()\nend\n```\n\nThat's it---now you can run the program with `mix run  `. To run the\nbundled example:\n\n```bash\n$ mix run examples/hello_world.exs\n```\n\nYou should see the content we created and be able to quit using 'q'.","ref":"under-the-hood.html#hello-world-from-scratch"},{"type":"extras","title":"Application Loops - Under the Hood","doc":"While the previous example illustrated the basics, it's unfortunately not a very\nuseful application on its own. Useful terminal applications need to update the\nview based on events or on a given interval (e.g., every second). They may also\nneed to hold state such as the cursor position or selected tab, and fetch data\nfrom local sources or via the network.","ref":"under-the-hood.html#application-loops"},{"type":"extras","title":"Rendering on an interval - Under the Hood","doc":"This time we'll build a clock application to show how intervals can be achieved.\nIt will display the current time and update each second.\n\nBe careful trying this one out, as we don't provide a way to quit yet---you'll\nneed to kill the process (for example, by closing your terminal window).\n\n```elixir\ndefmodule Clock do\n  alias Ratatouille.Window\n\n  import Ratatouille.View\n\n  def start do\n    {:ok, _pid} = Window.start_link()\n    loop()\n  end\n\n  def loop do\n    clock_view = render(DateTime.utc_now())\n    Window.update(clock_view)\n    Process.sleep(1_000)\n    loop()\n  end\n\n  def render(now) do\n    view do\n      panel title: \"Clock Example\" do\n        label(content: \"The time is: \" <> DateTime.to_string(now))\n      end\n    end\n  end\nend\n\nClock.start()\n```\n\nThere are a few things of note here.\n\nWe've defined this application in a module. This is how you'll usually want to\ndo it.\n\nWe've defined a `start/1` function to do the initial setup.\n\nThat setup calls the `loop/0` function, which is our application loop. Each loop\nrenders a view, updates the window, waits one second and then calls itself to\nstart the process all over again.\n\nThe view is built via a `render/1` function. If you're familiar with React.js,\nthe idea is similar. Our \"render\" functions should always be pure functions of\ntheir state--any two calls with the same arguments should always have the same\nresult. As such, we also pass in the state here: the current time.\n\nLastly, we remember to actually start this thing with `Clock.start()`.","ref":"under-the-hood.html#rendering-on-an-interval"},{"type":"extras","title":"Combining an interval with event handling - Under the Hood","doc":"In the clock example, we now have a working update interval, but no way to\nhandle events---and therefore, no way to quit the application. Let's fix that:\n\n```elixir\ndefmodule Clock do\n  alias Ratatouille.{EventManager, Window}\n\n  import Ratatouille.View\n\n  def start do\n    {:ok, _pid} = Window.start_link()\n    {:ok, _pid} = EventManager.start_link()\n    :ok = EventManager.subscribe(self())\n    loop()\n  end\n\n  def loop do\n    clock_view = render(DateTime.utc_now())\n    Window.update(clock_view)\n\n    receive do\n      {:event, %{ch: ?q}} ->\n        :ok = Window.close()\n    after\n      1_000 ->\n        loop()\n    end\n  end\n\n  def render(now) do\n    view do\n      panel title: \"Clock Example ('q' to quit)\" do\n        label(content: \"The time is: \" <> DateTime.to_string(now))\n      end\n    end\n  end\nend\n\nClock.start()\n```\n\nIn the new version, we fire up the event manager and subscribe ourself to\nany events that come through.\n\nThen we use a `receive` like in the hello world application, and pair that with\nan `after` to achieve the interval.\n\nInstead of sleeping, our loop now has a new job: wait for events for 1 second,\nthen start over (updating and re-rendering the clock).\n\nTry it out yourself:\n\n```bash\nmix run examples/clock.exs\n```","ref":"under-the-hood.html#combining-an-interval-with-event-handling"},{"type":"extras","title":"Holding on to state - Under the Hood","doc":"In the clock example, we were working with externally defined state that we\nretrieved in each loop with some help from Elixir and the BEAM.\n\nBut what if we need to hold on to state across loops?\n\nAs a concrete example, imagine we want to hold on to the cursor position. Maybe\nthis is a text editor. This is possible by storing our state within the\napplication loop itself (not unlike how a gen_server works under the hood):\n\n```elixir\ndef start do\n  # ...\n\n  loop(0) # initial cursor\nend\n\ndef loop(cursor) do\n  # ...\n\n  receive do\n    {:event, %{ch: @arrow_down}} ->\n      loop(cursor + 1)\n\n    {:event, %{ch: @arrow_up}} ->\n      loop(cursor - 1)\n  after\n    1_000 ->\n      loop(cursor)\n  end\nend\n```","ref":"under-the-hood.html#holding-on-to-state"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}